@@ -41,14 +41,14 @@ class Action(object):
             print_p = True
         dtemp = tempfile.mkdtemp(prefix='tryton_')
         fp_name = os.path.join(dtemp,
-                name.replace(os.sep, '_').replace(os.altsep or os.sep, '_') \
-                        + os.extsep + type)
+            name.replace(os.sep, '_').replace(os.altsep or os.sep, '_')
+            + os.extsep + type)
         with open(fp_name, 'wb') as file_d:
             file_d.write(data)
         if email_print:
             mailto(to=email.get('to'), cc=email.get('cc'),
-                    subject=email.get('subject'), body=email.get('body'),
-                    attachment=fp_name)
+                subject=email.get('subject'), body=email.get('body'),
+                attachment=fp_name)
         else:
             file_open(fp_name, type, print_p=print_p)
         return True
@@ -119,6 +119,12 @@ class Action(object):
             search_value = PYSONDecoder(search_context).decode(
                     action['pyson_search_value'] or '[]')
 
+            tab_domain_context = ctx.copy()
+            tab_domain_context['context'] = ctx
+            tab_domain_context['_user'] = rpc._USER
+            decoder = PYSONDecoder(tab_domain_context)
+            tab_domain = [(n, decoder.decode(d)) for n, d in action['domains']]
+
             name = False
             if action.get('window_name', True):
                 name = action.get('name', False)
@@ -131,7 +137,8 @@ class Action(object):
                     limit=action.get('limit'),
                     auto_refresh=action.get('auto_refresh'),
                     search_value=search_value,
-                    icon=(action.get('icon.rec_name') or ''))
+                    icon=(action.get('icon.rec_name') or ''),
+                    tab_domain=tab_domain)
         elif action['type'] == 'ir.action.wizard':
             Window.create_wizard(action['wiz_name'], data,
                 direct_print=action.get('direct_print', False),
@@ -154,13 +161,12 @@ class Action(object):
     def exec_keyword(keyword, data=None, context=None, warning=True,
             alwaysask=False):
         actions = []
-        if 'id' in data:
-            model_id = data.get('id', False)
-            try:
-                actions = RPCExecute('model', 'ir.action.keyword',
-                    'get_keyword', keyword, (data['model'], model_id))
-            except RPCException:
-                return False
+        model_id = data.get('id', False)
+        try:
+            actions = RPCExecute('model', 'ir.action.keyword',
+                'get_keyword', keyword, (data['model'], model_id))
+        except RPCException:
+            return False
 
         keyact = {}
         for action in actions:
@@ -194,11 +200,10 @@ class Action(object):
             encoder = PYSONEncoder()
             if 'pyson_domain' in action:
                 action['pyson_domain'] = encoder.encode(
-                    record.expr_eval(action['pyson_domain'], check_load=False))
+                    record.expr_eval(action['pyson_domain']))
             if 'pyson_context' in action:
                 action['pyson_context'] = encoder.encode(
-                    record.expr_eval(action['pyson_context'],
-                        check_load=False))
+                    record.expr_eval(action['pyson_context']))
 
         else:
             raise NotImplementedError("Action type '%s' is not supported" %
