@@ -27,8 +27,7 @@ class Button(gtk.Button):
 
     def state_set(self, record):
         if record:
-            states = record.expr_eval(self.attrs.get('states', {}),
-                check_load=False)
+            states = record.expr_eval(self.attrs.get('states', {}))
         else:
             states = {}
         if states.get('invisible', False):
