@@ -36,7 +36,6 @@ try:
     import ssl
 except ImportError:
     ssl = None
-import dis
 from threading import Lock
 try:
     import pytz
@@ -45,6 +44,7 @@ except ImportError:
 
 from tryton.exceptions import (TrytonServerError, TrytonError,
     TrytonServerUnavailable)
+from tryton.pyson import PYSONEncoder
 
 _ = gettext.gettext
 
@@ -170,6 +170,44 @@ class ModelAccess(object):
 MODELACCESS = ModelAccess()
 
 
+class ViewSearch(object):
+    searches = None
+
+    def load_searches(self):
+        try:
+            self.searches = rpc.execute('model', 'ir.ui.view_search',
+                'get_search', rpc.CONTEXT)
+        except TrytonServerError:
+            self.searches = {}
+
+    def __getitem__(self, model):
+        return self.searches.get(model, [])
+
+    def add(self, model, name, domain):
+        try:
+            id_, = RPCExecute('model', 'ir.ui.view_search',
+                'create', [{
+                        'model': model,
+                        'name': name,
+                        'domain': PYSONEncoder().encode(domain),
+                        }])
+        except RPCException:
+            return
+        self.searches.setdefault(model, []).append((id_, name, domain))
+
+    def remove(self, model, id_):
+        try:
+            RPCExecute('model', 'ir.ui.view_search', 'delete', [id_])
+        except RPCException:
+            return
+        for i, domain in enumerate(self.searches[model]):
+            if domain[0] == id_:
+                del self.searches[model][i]
+                break
+
+VIEW_SEARCH = ViewSearch()
+
+
 def find_in_path(name):
     if os.name == "nt":
         sep = ';'
@@ -435,7 +473,7 @@ def file_selection(title, filename='',
             filepath = filepath.decode('utf-8')
             try:
                 CONFIG['client.default_path'] = \
-                        os.path.dirname(filepath)
+                    os.path.dirname(filepath)
                 CONFIG.save()
             except IOError:
                 pass
@@ -448,7 +486,7 @@ def file_selection(title, filename='',
             filenames = [x.decode('utf-8') for x in filenames]
             try:
                 CONFIG['client.default_path'] = \
-                        os.path.dirname(filenames[0])
+                    os.path.dirname(filenames[0])
             except IOError:
                 pass
         parent.present()
@@ -963,8 +1001,8 @@ def send_bugtracker(title, msg):
                 # second create issue with this message
                 issue_id = server.create('issue', *['messages=' + str(msg_id),
                     'nosy=' + user, 'title=' + title, 'priority=bug'])
-                message(_('Created new bug with ID ') + \
-                        'issue%s' % issue_id)
+                message(_('Created new bug with ID ')
+                    + 'issue%s' % issue_id)
             webbrowser.open(CONFIG['roundup.url'] + 'issue%s' % issue_id,
                 new=2)
         except (socket.error, xmlrpclib.Fault), exception:
@@ -1001,7 +1039,7 @@ def process_exception(exception, *args, **kwargs):
             sys.exit()
         elif exception.faultCode == 'NotLogged':
             if rpc.CONNECTION is None:
-                message(_('Connection error!\n' \
+                message(_('Connection error!\n'
                         'Unable to connect to the server!'))
                 return False
     elif isinstance(exception, TrytonServerError):
@@ -1009,11 +1047,11 @@ def process_exception(exception, *args, **kwargs):
             name, msg, description = exception.args
             res = userwarning(description, msg)
             if res in ('always', 'ok'):
-                args2 = ('model', 'res.user.warning', 'create', {
-                        'user': rpc._USER,
-                        'name': name,
-                        'always': (res == 'always'),
-                        }, rpc.CONTEXT)
+                args2, = ('model', 'res.user.warning', 'create', [{
+                            'user': rpc._USER,
+                            'name': name,
+                            'always': (res == 'always'),
+                            }], rpc.CONTEXT)
                 try:
                     rpc_execute(*args2)
                 except TrytonServerError, exception:
@@ -1046,6 +1084,14 @@ def process_exception(exception, *args, **kwargs):
                 return False
         elif exception.faultCode == 'NotLogged':
             from tryton.gui.main import Main
+            if kwargs.get('session', rpc._SESSION) != rpc._SESSION:
+                if args:
+                    try:
+                        return rpc_execute(*args)
+                    except TrytonServerError, exception:
+                        return process_exception(exception, *args,
+                            rpc_execute=rpc_execute)
+                return
             if not PLOCK.acquire(False):
                 return False
             hostname = rpc._HOST
@@ -1060,7 +1106,7 @@ def process_exception(exception, *args, **kwargs):
                             rpc._DATABASE)
                     Main.get_main().refresh_ssl()
                     if res == -1:
-                        message(_('Connection error!\n' \
+                        message(_('Connection error!\n'
                                 'Unable to connect to the server!'))
                         return False
                     if res < 0:
@@ -1230,6 +1276,7 @@ class RPCProgress(object):
         return True
 
     def run(self, process_exception_p=True, main_iteration_p=True):
+        session = rpc._SESSION
         thread.start_new_thread(self.start, ())
 
         watch = None
@@ -1252,7 +1299,7 @@ class RPCProgress(object):
                     return RPCProgress('execute',
                         args).run(process_exception_p, main_iteration_p)
                 result = process_exception(self.exception, *self.args,
-                    rpc_execute=rpc_execute)
+                    rpc_execute=rpc_execute, session=session)
                 if result is False:
                     raise RPCException(self.exception)
                 return result
@@ -1423,60 +1470,6 @@ def filter_domain(domain):
             res.extend(filter_domain(arg))
     return res
 
-_ALLOWED_CODES = set(dis.opmap[x] for x in [
-        'POP_TOP', 'ROT_TWO', 'ROT_THREE', 'ROT_FOUR', 'DUP_TOP', 'BUILD_LIST',
-        'BUILD_MAP', 'BUILD_TUPLE', 'LOAD_CONST', 'RETURN_VALUE',
-        'STORE_SUBSCR', 'UNARY_POSITIVE', 'UNARY_NEGATIVE', 'UNARY_NOT',
-        'UNARY_INVERT', 'BINARY_POWER', 'BINARY_MULTIPLY', 'BINARY_DIVIDE',
-        'BINARY_FLOOR_DIVIDE', 'BINARY_TRUE_DIVIDE', 'BINARY_MODULO',
-        'BINARY_ADD', 'BINARY_SUBTRACT', 'BINARY_LSHIFT', 'BINARY_RSHIFT',
-        'BINARY_AND', 'BINARY_XOR', 'BINARY_OR', 'STORE_MAP', 'LOAD_NAME',
-        'CALL_FUNCTION', 'COMPARE_OP', 'LOAD_ATTR', 'STORE_NAME', 'GET_ITER',
-        'FOR_ITER', 'LIST_APPEND', 'JUMP_ABSOLUTE', 'DELETE_NAME',
-        'JUMP_IF_TRUE', 'JUMP_IF_FALSE', 'JUMP_IF_FALSE_OR_POP',
-        'JUMP_IF_TRUE_OR_POP', 'POP_JUMP_IF_FALSE', 'POP_JUMP_IF_TRUE',
-        'BINARY_SUBSCR',
-        ] if x in dis.opmap)
-
-_SAFE_EVAL_CACHE = {}
-
-
-def safe_eval(source, data=None):
-    if '__subclasses__' in source:
-        raise ValueError('__subclasses__ not allowed')
-    if hashlib:
-        key = hashlib.md5(source).digest()
-    else:
-        key = md5.new(source).digest()
-    c = _SAFE_EVAL_CACHE.get(key)
-    if not c:
-        c = compile(source, '', 'eval')
-        codes = []
-        s = c.co_code
-        i = 0
-        while i < len(s):
-            code = ord(s[i])
-            codes.append(code)
-            if code >= dis.HAVE_ARGUMENT:
-                i += 3
-            else:
-                i += 1
-        for code in codes:
-            if code not in _ALLOWED_CODES:
-                raise ValueError('opcode %s not allowed' % dis.opname[code])
-        if len(_SAFE_EVAL_CACHE) > 1024:
-            _SAFE_EVAL_CACHE.clear()
-        _SAFE_EVAL_CACHE[key] = c
-    return eval(c, {'__builtins__': {
-        'True': True,
-        'False': False,
-        'str': str,
-        'globals': locals,
-        'locals': locals,
-        'bool': bool,
-        'dict': dict,
-        }}, data)
-
 
 def timezoned_date(date, reverse=False):
     if pytz and rpc.CONTEXT.get('timezone') and rpc.TIMEZONE:
