@@ -87,10 +87,10 @@ def group_operator(tokens):
             yield cur + nex
             cur = None
         else:
-            if cur:
+            if cur is not None:
                 yield cur
             cur = nex
-    if cur:
+    if cur is not None:
         yield cur
 
 
@@ -220,8 +220,8 @@ def convert_value(field, value):
                         date_format() + ' ' + time_format(field))[:6]))
         except ValueError:
             try:
-                return untimezoned_date(datetime.datetime(*time.strptime(value,
-                            date_format())[:6]))
+                return datetime.datetime(*time.strptime(value,
+                        date_format())[:6])
             except ValueError:
                 return
         except TypeError:
@@ -396,7 +396,7 @@ def format_value(field, value):
             digit = int(field.get('digits', (16, 2))[1])
         except ValueError:
             digit = 2
-        return locale.format('%.' + str(digit) + 'f', value or 0.0, True)
+        return locale.format('%.*f', (digit, value or 0.0), True)
 
     def format_selection():
         selections = dict(field['selection'])
@@ -440,7 +440,8 @@ def format_value(field, value):
         }
     if isinstance(value, (list, tuple)):
         return ';'.join(format_value(field, x) for x in value)
-    return quote(converts.get(field['type'], lambda: value)())
+    return quote(converts.get(field['type'],
+            lambda: value if value is not None else '')())
 
 
 def test_format_boolean():
@@ -569,9 +570,15 @@ def complete_value(field, value):
             yield True
 
     def complete_selection():
+        test_value = value if value is not None else ''
+        if isinstance(value, list):
+            test_value = value[-1]
         for svalue, test in field['selection']:
-            if test.lower().startswith(value.lower()):
-                yield svalue
+            if test.lower().startswith(test_value.lower()):
+                if isinstance(value, list):
+                    yield value[:-1] + [svalue]
+                else:
+                    yield svalue
 
     def complete_datetime():
         yield datetime.date.today()
@@ -594,6 +601,24 @@ def complete_value(field, value):
     return completes.get(field['type'], lambda: [])()
 
 
+def test_complete_selection():
+    field = {
+        'type': 'selection',
+        'selection': [
+            ('male', 'Male'),
+            ('female', 'Female'),
+            ],
+        }
+    for value, result in (
+            ('m', ['male']),
+            ('test', []),
+            ('', ['male', 'female']),
+            (None, ['male', 'female']),
+            (['male', 'f'], [['male', 'female']]),
+            ):
+        assert list(complete_value(field, value)) == result
+
+
 def parenthesize(tokens):
     "Nest tokens according to parenthesis"
     for token in tokens:
@@ -672,7 +697,7 @@ def test_operatorize():
             (['a', 'or', iter(['b', 'c'])], [['OR', 'a', ['b', 'c']]]),
             (['a', 'or', iter(['b', 'c']), 'd'],
                 [['OR', 'a', ['b', 'c']], 'd']),
-            (['a', iter(['b', 'c']),  'or', 'd'],
+            (['a', iter(['b', 'c']), 'or', 'd'],
                 ['a', ['OR', ['b', 'c'], 'd']]),
             (['a', 'or', iter(['b', 'or', 'c'])],
                 [['OR', 'a', [['OR', 'b', 'c']]]]),
@@ -747,7 +772,9 @@ class DomainParser(object):
                             operator = '!'
                         value = value.replace('%%', '%')
                 def_operator = default_operator(field)
-                if def_operator in operator:
+                if (def_operator == operator.strip()
+                        or (def_operator in operator
+                            and 'not' in operator)):
                     operator = operator.rstrip(def_operator
                         ).replace('not', '!').strip()
                 if operator.endswith('in'):
@@ -755,8 +782,13 @@ class DomainParser(object):
                         operator = '!'
                     else:
                         operator = ''
+                formatted_value = format_value(field, value)
+                if (operator in OPERATORS and
+                        field['type'] in ('char', 'text', 'sha', 'selection')
+                        and value == ''):
+                    formatted_value = '""'
                 return '%s: %s%s' % (quote(field['string']), operator,
-                    format_value(field, value))
+                    formatted_value)
             else:
                 return '(%s)' % self.string(clause)
 
@@ -891,7 +923,7 @@ class DomainParser(object):
                         if lvalue:
                             yield name + (lvalue,)
                         else:
-                            yield name + ('',)
+                            yield name + (None,)
                     break
 
         parts = []
@@ -932,8 +964,9 @@ class DomainParser(object):
                         operator = negate_operator(default_operator(field))
                     if 'like' in operator:
                         value = likify(value)
-                    if field['type'] in ('integer', 'float', 'numeric'):
-                        if '..' in value:
+                    if field['type'] in ('integer', 'float', 'numeric',
+                            'datetime', 'date', 'time'):
+                        if value and '..' in value:
                             lvalue, rvalue = value.split('..', 1)
                             lvalue = convert_value(field, lvalue)
                             rvalue = convert_value(field, rvalue)
@@ -942,7 +975,10 @@ class DomainParser(object):
                                     (field['name'], '<', rvalue),
                                     ])
                             continue
-                    value = convert_value(field, value)
+                    if isinstance(value, list):
+                        value = [convert_value(field, v) for v in value]
+                    else:
+                        value = convert_value(field, value)
                     yield field['name'], operator, value
 
 
@@ -956,8 +992,15 @@ def test_string():
                 'string': '(Sur)Name',
                 'type': 'char',
                 },
+            'date': {
+                'string': 'Date',
+                'type': 'date',
+                },
             })
     assert dom.string([('name', '=', 'Doe')]) == 'Name: =Doe'
+    assert dom.string([('name', '=', None)]) == 'Name: ='
+    assert dom.string([('name', '=', '')]) == 'Name: =""'
+    assert dom.string([('name', 'ilike', '%')]) == 'Name: '
     assert dom.string([('name', 'ilike', '%Doe%')]) == 'Name: Doe'
     assert dom.string([('name', 'ilike', 'Doe')]) == 'Name: =Doe'
     assert dom.string([('name', 'ilike', 'Doe%')]) == 'Name: Doe%'
@@ -980,9 +1023,11 @@ def test_string():
             ['OR',
                 ('name', 'ilike', '%John%'),
                 ('name', 'ilike', '%Jane%')]]) == \
-                    'Name: Doe (Name: John or Name: Jane)'
+        'Name: Doe (Name: John or Name: Jane)'
     assert dom.string([]) == ''
     assert dom.string([('surname', 'ilike', '%Doe%')]) == '"(Sur)Name": Doe'
+    assert dom.string([('date', '>=', datetime.date(2012, 10, 24))]) == \
+        'Date: >=10/24/2012'
 
 
 def test_group():
@@ -1053,7 +1098,20 @@ def test_group():
         ('Name', None, 'Doe'),
         ]
     assert rlist(dom.group(udlex(u'Name:'))) == [
-        ('Name', None, ''),
+        ('Name', None, None),
+        ]
+    assert rlist(dom.group(udlex(u'Name: ='))) == [
+        ('Name', '=', None),
+        ]
+    assert rlist(dom.group(udlex(u'Name: =""'))) == [
+        ('Name', '=', ''),
+        ]
+    assert rlist(dom.group(udlex(u'Name: = ""'))) == [
+        ('Name', '=', ''),
+        ]
+    assert rlist(dom.group(udlex(u'Name: = Name: Doe'))) == [
+        ('Name', '=', None),
+        ('Name', None, 'Doe'),
         ]
 
 
@@ -1069,9 +1127,24 @@ def test_parse_clause():
                 'name': 'integer',
                 'type': 'integer',
                 },
+            'selection': {
+                'string': 'Selection',
+                'name': 'selection',
+                'type': 'selection',
+                'selection': [
+                    ('male', 'Male'),
+                    ('female', 'Female'),
+                    ],
+                },
             })
     assert rlist(dom.parse_clause([('John',)])) == [
         ('rec_name', 'ilike', '%John%')]
+    assert rlist(dom.parse_clause([('Name', None, None)])) == [
+        ('name', 'ilike', '%')]
+    assert rlist(dom.parse_clause([('Name', '=', None)])) == [
+        ('name', '=', None)]
+    assert rlist(dom.parse_clause([('Name', '=', '')])) == [
+        ('name', '=', '')]
     assert rlist(dom.parse_clause([('Name', None, 'Doe')])) == [
         ('name', 'ilike', '%Doe%')]
     assert rlist(dom.parse_clause([('Name', '!', 'Doe')])) == [
@@ -1082,6 +1155,13 @@ def test_parse_clause():
     assert rlist(dom.parse_clause([('Name', '!', ['John', 'Jane'])])) == [
         ('name', 'not in', ['John', 'Jane']),
         ]
+    assert rlist(dom.parse_clause([('Selection', None, ['Male', 'Female'])])) \
+        == [
+            ('selection', 'in', ['male', 'female'])
+            ]
+    assert rlist(dom.parse_clause([('Integer', None, None)])) == [
+        ('integer', '=', None),
+        ]
     assert rlist(dom.parse_clause([('Integer', None, '3..5')])) == [[
             ('integer', '>=', 3),
             ('integer', '<', 5),
