@@ -4,7 +4,6 @@
 import gettext
 import gtk
 import gobject
-import locale
 from tryton.gui.window.view_form.screen import Screen
 from tryton.action import Action
 from tryton.gui import Main
@@ -13,11 +12,11 @@ from tryton.gui.window.win_export import WinExport
 from tryton.gui.window.win_import import WinImport
 from tryton.gui.window.attachment import Attachment
 from tryton.signal_event import SignalEvent
-from tryton.common import (TRYTON_ICON, message, sur, sur_3b, COLOR_SCHEMES,
-    timezoned_date)
+from tryton.common import message, sur, sur_3b, COLOR_SCHEMES, timezoned_date
 import tryton.common as common
 from tryton.translate import date_format
 from tryton.common import RPCExecute, RPCException
+from tryton import plugins
 
 from tabcontent import TabContent
 
@@ -62,7 +61,6 @@ class Form(SignalEvent, TabContent):
         (_('_Next'), 'tryton-go-next', 'sig_next', '<tryton>/Form/Next'),
         (_('_Search'), 'tryton-find', 'sig_search', '<tryton>/Form/Search'),
         (_('View _Logs...'), None, 'sig_logs', None),
-        (_('_Go to Record ID...'), None, 'sig_goto', '<tryton>/Form/Goto'),
         (None,) * 4,
         (_('_Close Tab'), 'tryton-close', 'sig_win_close',
             '<tryton>/Form/Close'),
@@ -89,7 +87,7 @@ class Form(SignalEvent, TabContent):
 
     def __init__(self, model, res_id=False, domain=None, mode=None,
             view_ids=None, context=None, name=False, limit=None,
-            auto_refresh=False, search_value=None):
+            auto_refresh=False, search_value=None, tab_domain=None):
         super(Form, self).__init__()
 
         if not mode:
@@ -112,7 +110,8 @@ class Form(SignalEvent, TabContent):
 
         self.screen = Screen(self.model, mode=mode, context=self.context,
             view_ids=view_ids, domain=domain, limit=limit,
-            readonly=bool(auto_refresh), search_value=search_value)
+            readonly=bool(auto_refresh), search_value=search_value,
+            tab_domain=tab_domain)
         self.screen.widget.show()
 
         if not name:
@@ -177,49 +176,6 @@ class Form(SignalEvent, TabContent):
             and self.auto_refresh == value.auto_refresh
             and self.screen.search_value == value.screen.search_value)
 
-    def sig_goto(self, widget=None):
-        if not self.modified_save():
-            return
-        parent = common.get_toplevel_window()
-        win = gtk.Dialog(_('Go to ID'), parent,
-            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK,
-                gtk.RESPONSE_OK))
-        win.set_icon(TRYTON_ICON)
-        win.set_has_separator(True)
-        win.set_default_response(gtk.RESPONSE_OK)
-
-        table = gtk.Table(2, 2)
-        table.attach(gtk.Label(_('Go to ID:')), 1, 2, 0, 1, gtk.FILL)
-        img = gtk.Image()
-        img.set_from_stock('tryton-go-jump', gtk.ICON_SIZE_DIALOG)
-        table.attach(img, 0, 1, 0, 2, gtk.FILL)
-
-        entry = gtk.Entry()
-        entry.set_property('activates_default', True)
-        entry.set_max_length(0)
-        entry.set_alignment(1.0)
-
-        def sig_insert_text(widget, new_text, new_text_length, position):
-            value = widget.get_text()
-            position = widget.get_position()
-            new_value = value[:position] + new_text + value[position:]
-            try:
-                locale.atoi(new_value)
-            except ValueError:
-                widget.stop_emission('insert-text')
-        entry.connect('insert_text', sig_insert_text)
-        table.attach(entry, 1, 2, 1, 2)
-
-        win.vbox.pack_start(table, expand=True, fill=True)
-        win.show_all()
-
-        response = win.run()
-        if response == gtk.RESPONSE_OK:
-            self.screen.display(locale.atoi(entry.get_text()), set_cursor=True)
-        win.destroy()
-        parent.present()
-
     def destroy(self):
         self.screen.destroy()
         self.screen = None
@@ -382,7 +338,7 @@ class Form(SignalEvent, TabContent):
         if not hasattr(self, 'screen'):
             return False
         if test_modified and self.screen.modified():
-            res = sur_3b(_('This record has been modified\n' \
+            res = sur_3b(_('This record has been modified\n'
                     'do you want to save it ?'))
             if res == 'ok':
                 self.sig_save(None)
@@ -494,8 +450,8 @@ class Form(SignalEvent, TabContent):
         self.screen.current_view.set_value()
         if self.screen.modified():
             value = sur_3b(
-                    _('This record has been modified\n' \
-                            'do you want to save it ?'))
+                _('This record has been modified\n'
+                    'do you want to save it ?'))
             if value == 'ok':
                 return self.sig_save(None)
             if value == 'ko':
@@ -554,7 +510,7 @@ class Form(SignalEvent, TabContent):
                 tbutton.connect('toggled', self.action_popup)
                 self.tooltips.set_tip(tbutton, tooltip)
                 self.buttons[special_action] = tbutton
-                tbutton._can_be_sensitive = bool(toolbars.get(action_type))
+                tbutton._can_be_sensitive = bool(tbutton._menu.get_children())
             else:
                 tbutton = gtk.SeparatorToolItem()
             gtktoolbar.insert(tbutton, -1)
@@ -579,7 +535,18 @@ class Form(SignalEvent, TabContent):
             menuitem.connect('activate', self._popup_menu_selected, widget,
                 new_action, keyword)
             menu.add(menuitem)
-
+        if keyword == 'action':
+            menu.add(gtk.SeparatorMenuItem())
+            for plugin in plugins.MODULES:
+                for name, func in plugin.get_plugins(self.model):
+                    menuitem = gtk.MenuItem('_' + name)
+                    menuitem.set_use_underline(True)
+                    menuitem.connect('activate', lambda m, func: func({
+                                'model': self.model,
+                                'ids': self.id_get(),
+                                'id': self.id_get(),
+                                }), func)
+                    menu.add(menuitem)
         return menu
 
     def _popup_menu_selected(self, menuitem, togglebutton, action, keyword):
