@@ -111,7 +111,7 @@ class Action(SignalEvent):
 
     def _get_active(self):
         if self.screen and self.screen.current_record:
-            return common.EvalEnvironment(self.screen.current_record, False)
+            return common.EvalEnvironment(self.screen.current_record)
 
     active = property(_get_active)
 
@@ -129,4 +129,6 @@ class Action(SignalEvent):
         del self.domain[:]
         self.domain.extend(new_domain)
         if hasattr(self, 'screen'):  # Catch early update
-            self.display()
+            # Using idle_add to prevent corruption of the event who triggered
+            # the update.
+            gtk.idle_add(self.display)
