@@ -35,8 +35,6 @@ class CharField(object):
     def __init__(self, attrs):
         self.attrs = attrs
         self.name = attrs['name']
-        self.internal = False
-        self.default_attrs = {}
 
     def sig_changed(self, record):
         if self.get_state_attrs(record).get('readonly', False):
@@ -48,7 +46,7 @@ class CharField(object):
 
     def domains_get(self, record):
         screen_domain = domain_inversion(record.group.domain4inversion,
-            self.name, EvalEnvironment(record, False))
+            self.name, EvalEnvironment(record))
         if isinstance(screen_domain, bool) and not screen_domain:
             screen_domain = [('id', '=', False)]
         elif isinstance(screen_domain, bool) and screen_domain:
@@ -68,13 +66,11 @@ class CharField(object):
         else:
             return screen_domain, screen_domain
 
-    def context_get(self, record, check_load=True, eval_context=True):
+    def context_get(self, record):
         context = record.context_get().copy()
         if record.parent:
             context.update(record.parent.context_get())
-        if eval_context:
-            context.update(record.expr_eval(self.attrs.get('context', {}),
-                check_load=check_load))
+        context.update(record.expr_eval(self.attrs.get('context', {})))
         return context
 
     def check_required(self, record):
@@ -122,27 +118,21 @@ class CharField(object):
                 if setdefault:
                     self.set_client(record, value)
                     self.get_state_attrs(record)['domain_readonly'] = True
-            res = res and eval_domain(domain, EvalEnvironment(record, False))
+            res = res and eval_domain(domain, EvalEnvironment(record))
         self.get_state_attrs(record)['valid'] = res
         return res
 
-    def set(self, record, value, modified=False):
+    def set(self, record, value):
         record.value[self.name] = value
-        if modified:
-            record.modified_fields.setdefault(self.name)
-            record.signal('record-modified')
-            record.signal('record-changed')
-        return True
 
-    def get(self, record, check_load=True, readonly=True, modified=False):
+    def get(self, record):
         return record.value.get(self.name) or self._default
 
-    def get_eval(self, record, check_load=True):
-        return self.get(record, check_load=check_load, readonly=True,
-                modified=False)
+    def get_eval(self, record):
+        return self.get(record)
 
-    def get_on_change_value(self, record, check_load=True):
-        return self.get_eval(record, check_load=check_load)
+    def get_on_change_value(self, record):
+        return self.get_eval(record)
 
     def set_client(self, record, value, force_change=False):
         previous_value = self.get(record)
@@ -161,17 +151,16 @@ class CharField(object):
     def get_client(self, record):
         return record.value.get(self.name) or self._default
 
-    def set_default(self, record, value, modified=False):
-        res = self.set(record, value, modified=modified)
-        return res
+    def set_default(self, record, value):
+        self.set(record, value)
+        record.modified_fields.setdefault(self.name)
 
     def set_on_change(self, record, value):
         record.modified_fields.setdefault(self.name)
-        return self.set(record, value, modified=False)
+        return self.set(record, value)
 
     def state_set(self, record, states=('readonly', 'required', 'invisible')):
-        state_changes = record.expr_eval(self.attrs.get('states', {}),
-                check_load=False)
+        state_changes = record.expr_eval(self.attrs.get('states', {}))
         for key in states:
             if key == 'readonly' and self.attrs.get(key, False):
                 continue
@@ -180,12 +169,8 @@ class CharField(object):
             elif key in self.attrs:
                 self.get_state_attrs(record)[key] = self.attrs[key]
         if (record.group.readonly
-            or self.get_state_attrs(record).get('domain_readonly')):
+                or self.get_state_attrs(record).get('domain_readonly')):
             self.get_state_attrs(record)['readonly'] = True
-        if 'value' in state_changes:
-            value = state_changes['value']
-            if value:
-                self.set(record, value, modified=True)
 
     def get_state_attrs(self, record):
         if self.name not in record.state_attrs:
@@ -256,6 +241,8 @@ class DateField(CharField):
 
 class TimeField(CharField):
 
+    _default = None
+
     def set_client(self, record, value, force_change=False):
         if not isinstance(value, datetime.time):
             try:
@@ -287,7 +274,7 @@ class NumberField(CharField):
                 return False
         return True
 
-    def get(self, record, check_load=True, readonly=True, modified=False):
+    def get(self, record):
         return record.value.get(self.name, self._default)
 
     def digits(self, record):
@@ -311,7 +298,7 @@ class FloatField(NumberField):
         value = record.value.get(self.name)
         if value is not None:
             digits = self.digits(record)
-            return locale.format('%.' + str(digits[1]) + 'f', value, True)
+            return locale.format('%.*f', (digits[1], value), True)
         else:
             return ''
 
@@ -331,7 +318,7 @@ class NumericField(NumberField):
         value = record.value.get(self.name)
         if value is not None:
             digits = self.digits(record)
-            return locale.format('%.' + str(digits[1]) + 'f', value, True)
+            return locale.format('%.*f', (digits[1], value), True)
         else:
             return ''
 
@@ -366,7 +353,7 @@ class BooleanField(CharField):
         super(BooleanField, self).set_client(record, value,
             force_change=force_change)
 
-    def get(self, record, check_load=True, readonly=True, modified=False):
+    def get(self, record):
         return bool(record.value.get(self.name))
 
     def get_client(self, record):
@@ -380,7 +367,7 @@ class M2OField(CharField):
 
     _default = None
 
-    def get(self, record, check_load=True, readonly=True, modified=False):
+    def get(self, record):
         value = record.value.get(self.name)
         if (record.parent_name == self.name
                 and self.attrs['relation'] == record.group.parent.model_name):
@@ -406,7 +393,7 @@ class M2OField(CharField):
         super(M2OField, self).set_client(record, value,
             force_change=force_change)
 
-    def set(self, record, value, modified=False):
+    def set(self, record, value):
         rec_name = record.value.get(self.name + '.rec_name') or ''
         if value is False:
             value = None
@@ -421,7 +408,7 @@ class M2OField(CharField):
         if not rec_name and value >= 0:
             try:
                 result, = RPCExecute('model', self.attrs['relation'], 'read',
-                    [value], ['rec_name'])
+                    [value], ['rec_name'], main_iteration=False)
             except RPCException:
                 return False
             rec_name = result['rec_name'] or ''
@@ -432,17 +419,12 @@ class M2OField(CharField):
             if record.parent:
                 if 'rec_name' not in record.parent.value:
                     record.parent.value['rec_name'] = rec_name
-        if modified:
-            record.modified_fields.setdefault(self.name)
-            record.signal('record-modified')
-            record.signal('record-changed')
 
-    def context_get(self, record, check_load=True, eval_context=True):
-        context = super(M2OField, self).context_get(record,
-                check_load=check_load, eval_context=eval_context)
-        if eval_context and self.attrs.get('datetime_field'):
+    def context_get(self, record):
+        context = super(M2OField, self).context_get(record)
+        if self.attrs.get('datetime_field'):
             context['_datetime'] = record.get_eval(
-                    check_load=check_load)[self.attrs.get('datetime_field')]
+                )[self.attrs.get('datetime_field')]
         return context
 
     def validation_domains(self, record):
@@ -528,28 +510,29 @@ class O2MField(CharField):
         self._set_default_value(record)
         return record.value.get(self.name)
 
-    def get(self, record, check_load=True, readonly=True, modified=False):
+    def get(self, record):
         if record.value.get(self.name) is None:
             return []
         record_removed = record.value[self.name].record_removed
         record_deleted = record.value[self.name].record_deleted
         result = [('add', [])]
         parent_name = self.attrs.get('relation_field', '')
+        to_create = []
         for record2 in record.value[self.name]:
             if record2 in record_removed or record2 in record_deleted:
                 continue
             if record2.id >= 0:
-                values = record2.get(check_load=check_load,
-                    get_readonly=readonly, get_modifiedonly=modified)
+                values = record2.get()
                 values.pop(parent_name, None)
                 if record2.modified and values:
                     result.append(('write', [record2.id], values))
                 result[0][1].append(record2.id)
             else:
-                values = record2.get(check_load=check_load,
-                    get_readonly=readonly)
+                values = record2.get()
                 values.pop(parent_name, None)
-                result.append(('create', values))
+                to_create.append(values)
+        if to_create:
+            result.append(('create', to_create))
         if not result[0][1]:
             del result[0]
         if record_removed:
@@ -568,7 +551,7 @@ class O2MField(CharField):
             result.update(record2.get_timestamp())
         return result
 
-    def get_eval(self, record, check_load=True):
+    def get_eval(self, record):
         if record.value.get(self.name) is None:
             return []
         record_removed = record.value[self.name].record_removed
@@ -576,55 +559,23 @@ class O2MField(CharField):
         return [x.id for x in record.value[self.name]
             if x not in record_removed and x not in record_deleted]
 
-    def get_on_change_value(self, record, check_load=True):
+    def get_on_change_value(self, record):
         result = []
         if record.value.get(self.name) is None:
             return []
         for record2 in record.value[self.name]:
             if not (record2.deleted or record2.removed):
                 result.append(
-                    record2.get_on_change_value(check_load=check_load))
+                    record2.get_on_change_value())
         return result
 
-    def set(self, record, value, modified=False):
-        from group import Group
-        group = record.value.get(self.name)
-        fields = {}
-        if group is not None:
-            fields = group.fields.copy()
-            # Unconnect to prevent infinite loop
-            group.signal_unconnect(group)
-            group.destroy()
-        elif record.model_name == self.attrs['relation']:
-            fields = record.group.fields
-        parent_name = self.attrs.get('relation_field', '')
-        group = Group(self.attrs['relation'], {},
-                parent=record, parent_name=parent_name,
-                child_name=self.name,
-                context=self.context,
-                parent_datetime_field=self.attrs.get('datetime_field'))
-        group.fields = fields
-        record.value[self.name] = group
-        group.load(value, display=False)
-        group.signal_connect(group, 'group-changed', self._group_changed)
-        group.signal_connect(group, 'group-list-changed',
-            self._group_list_changed)
-        group.signal_connect(group, 'group-cleared', self._group_cleared)
-        group.signal_connect(group, 'record-modified', self._record_modified)
-        if modified:
-            record.modified_fields.setdefault(self.name)
-            record.signal('record-modified')
-            record.signal('record-changed')
-
-    def set_client(self, record, value, force_change=False):
-        pass
-
-    def set_default(self, record, value, modified=False):
+    def _set_value(self, record, value, default=False):
         from group import Group
 
-        # value is a list of id
-        if value and len(value) and isinstance(value[0], (int, long)):
-            return self.set(record, value, modified=modified)
+        if not value or (len(value) and isinstance(value[0], (int, long))):
+            mode = 'list ids'
+        else:
+            mode = 'list values'
 
         group = record.value.get(self.name)
         fields = {}
@@ -638,63 +589,81 @@ class O2MField(CharField):
         if fields:
             fields = dict((fname, field.attrs)
                 for fname, field in fields.iteritems())
-        if value and len(value):
+        if mode == 'list values' and len(value):
             context = self.context_get(record)
-            field_names = []
-            for val in value:
-                for fieldname in val.keys():
-                    if (fieldname not in field_names
-                            and fieldname not in fields):
-                        field_names.append(fieldname)
+            field_names = set(f for v in value for f in v if f not in fields)
             if field_names:
                 try:
                     fields.update(RPCExecute('model', self.attrs['relation'],
-                            'fields_get', field_names, context=context))
+                            'fields_get', list(field_names),
+                            main_iteration=False, context=context))
                 except RPCException:
-                    return False
+                    return
 
         parent_name = self.attrs.get('relation_field', '')
         group = Group(self.attrs['relation'], fields,
-                parent=record, parent_name=parent_name, child_name=self.name,
-                context=self.context,
-                parent_datetime_field=self.attrs.get('datetime_field'))
-        if record.value.get(self.name):
-            group.record_deleted.extend(x for x in record.value[self.name]
-                if x.id >= 0)
-            group.record_deleted.extend(record.value[self.name].record_deleted)
-            group.record_removed.extend(record.value[self.name].record_removed)
+            parent=record, parent_name=parent_name,
+            child_name=self.name,
+            context=self.context,
+            parent_datetime_field=self.attrs.get('datetime_field'))
         record.value[self.name] = group
-        for vals in (value or []):
-            new_record = record.value[self.name].new(default=False)
-            new_record.set_default(vals, modified=modified)
-            group.add(new_record)
+        if mode == 'list ids':
+            group.load(value)
+        else:
+            for vals in value:
+                new_record = record.value[self.name].new(default=False)
+                if default:
+                    new_record.set_default(vals)
+                    group.add(new_record)
+                else:
+                    new_record.id *= -1  # Don't consider record as unsaved
+                    new_record.set(vals)
+                    group.append(new_record)
         group.signal_connect(group, 'group-changed', self._group_changed)
         group.signal_connect(group, 'group-list-changed',
             self._group_list_changed)
         group.signal_connect(group, 'group-cleared', self._group_cleared)
         group.signal_connect(group, 'record-modified', self._record_modified)
-        return True
+        return group
+
+    def set(self, record, value):
+        self._set_value(record, value, default=False)
+
+    def set_client(self, record, value, force_change=False):
+        pass
+
+    def set_default(self, record, value):
+        previous_group = record.value.get(self.name)
+        group = self._set_value(record, value, default=True)
+        if previous_group:
+            group.record_deleted.extend(x for x in previous_group if x.id >= 0)
+            group.record_deleted.extend(previous_group.record_deleted)
+            group.record_removed.extend(previous_group.record_removed)
+        record.modified_fields.setdefault(self.name)
 
     def set_on_change(self, record, value):
         self._set_default_value(record)
         if isinstance(value, (list, tuple)):
-            self.set(record, value, modified=False)
+            self._set_value(record, value)
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
             return True
 
         if value and (value.get('add') or value.get('update')):
             context = self.context_get(record)
-            field_names = []
-            for val in (value.get('add', []) + value.get('update', [])):
-                for fieldname in val.keys():
-                    if fieldname not in field_names:
-                        field_names.append(fieldname)
-            try:
-                fields = RPCExecute('model', self.attrs['relation'],
-                    'fields_get', field_names, context=context)
-            except RPCException:
-                return False
+            fields = record.value[self.name].fields
+            field_names = set(f for v in (
+                    value.get('add', []) + value.get('update', []))
+                for f in v if f not in fields and f != 'id')
+            if field_names:
+                try:
+                    fields = RPCExecute('model', self.attrs['relation'],
+                        'fields_get', list(field_names), main_iteration=False,
+                        context=context)
+                except RPCException:
+                    return False
+            else:
+                fields = {}
 
         to_remove = []
         for record2 in record.value[self.name]:
@@ -707,21 +676,21 @@ class O2MField(CharField):
                     to_remove.append(record2)
         for record2 in to_remove:
             record.value[self.name].remove(record2, signal=False,
-                force_remove=True)
+                force_remove=False)
 
-        if value and value.get('add') or value.get('update', []):
+        if value and (value.get('add') or value.get('update', [])):
             record.value[self.name].add_fields(fields, signal=False)
             for vals in value.get('add', []):
                 new_record = record.value[self.name].new(default=False)
                 record.value[self.name].add(new_record)
-                new_record.set(vals, modified=True, signal=False)
+                new_record.set_on_change(vals)
 
             for vals in value.get('update', []):
                 if 'id' not in vals:
                     continue
                 record2 = record.value[self.name].get(vals['id'])
                 if record2 is not None:
-                    record2.set(vals, modified=True, signal=False)
+                    record2.set_on_change(vals)
         return True
 
     def validation_domains(self, record):
@@ -733,9 +702,9 @@ class O2MField(CharField):
             return True
         res = True
         for record2 in record.value.get(self.name, []):
-            if not record2.loaded:
+            if not record2.loaded and record2.id >= 0:
                 continue
-            if not record2.validate():
+            if not record2.validate(softvalidation=softvalidation):
                 if not record2.modified:
                     record.value[self.name].remove(record2)
                 else:
@@ -761,7 +730,7 @@ class O2MField(CharField):
 
 class M2MField(O2MField):
 
-    def set(self, record, value, modified=False):
+    def set(self, record, value):
         from group import Group
         group = record.value.get(self.name)
         fields = {}
@@ -784,19 +753,15 @@ class M2MField(O2MField):
             group.record_removed.extend(record.value[self.name].record_removed)
         record.value[self.name] = group
         group.fields = fields
-        group.load(value, display=False)
+        group.load(value)
         group.signal_connect(group, 'group-changed', self._group_changed)
         group.signal_connect(group, 'group-list-changed',
             self._group_list_changed)
         group.signal_connect(group, 'group-cleared', self._group_cleared)
         group.signal_connect(group, 'record-modified', self._record_modified)
-        if modified:
-            record.modified_fields.setdefault(self.name)
-            record.signal('record-modified')
-            record.signal('record-changed')
 
-    def get_on_change_value(self, record, check_load=True):
-        return self.get_eval(record, check_load=check_load)
+    def get_on_change_value(self, record):
+        return self.get_eval(record)
 
 
 class ReferenceField(CharField):
@@ -811,7 +776,7 @@ class ReferenceField(CharField):
         else:
             return None
 
-    def get(self, record, check_load=True, readonly=True, modified=False):
+    def get(self, record):
         if (record.value.get(self.name)
                 and record.value[self.name][0]
                 and record.value[self.name][1] >= 0):
@@ -835,7 +800,7 @@ class ReferenceField(CharField):
         super(ReferenceField, self).set_client(record, value,
             force_change=force_change)
 
-    def set(self, record, value, modified=False):
+    def set(self, record, value):
         if not value:
             record.value[self.name] = self._default
             return
@@ -855,7 +820,7 @@ class ReferenceField(CharField):
             if not rec_name and ref_id >= 0:
                 try:
                     result, = RPCExecute('model', ref_model, 'read', [ref_id],
-                        ['rec_name'])
+                        ['rec_name'], main_iteration=False)
                 except RPCException:
                     return
                 rec_name = result['rec_name']
@@ -865,16 +830,13 @@ class ReferenceField(CharField):
             rec_name = ref_id
         record.value[self.name] = ref_model, ref_id
         record.value[self.name + '.rec_name'] = rec_name
-        if modified:
-            record.modified_fields.setdefault(self.name)
-            record.signal('record-modified')
 
 
 class BinaryField(CharField):
 
     _default = None
 
-    def get(self, record, check_load=True, readonly=True, modified=False):
+    def get(self, record):
         result = record.value.get(self.name) or self._default
         if isinstance(result, basestring):
             try:
@@ -913,7 +875,8 @@ class BinaryField(CharField):
             context = record.context_get()
             try:
                 values, = RPCExecute('model', record.model_name, 'read',
-                    [record.id], [self.name], context=context)
+                    [record.id], [self.name], main_iteration=False,
+                    context=context)
             except RPCException:
                 return ''
             _, filename = tempfile.mkstemp(prefix='tryton_')
@@ -922,6 +885,19 @@ class BinaryField(CharField):
             self.set(record, filename)
         return self.get(record)
 
+
+class DictField(CharField):
+
+    _default = {}
+
+    def validation_domains(self, record):
+        screen_domain, attr_domain = self.domains_get(record)
+        return screen_domain, screen_domain
+
+    def domain_get(self, record):
+        screen_domain, attr_domain = self.domains_get(record)
+        return localize_domain(inverse_leaf(screen_domain)) + attr_domain
+
 TYPES = {
     'char': CharField,
     'sha': CharField,
@@ -941,4 +917,5 @@ TYPES = {
     'time': TimeField,
     'one2one': O2OField,
     'binary': BinaryField,
+    'dict': DictField,
 }
