@@ -48,14 +48,19 @@ class Record(SignalEvent):
                         (field.attrs.get('loading', 'eager')
                             for field in self.group.fields.itervalues()),
                         'eager')
+                # Set a valid name for next loaded check
+                for fname, field in self.group.fields.iteritems():
+                    if field.attrs.get('loading', 'eager') == loading:
+                        name = fname
+                        break
             else:
                 loading = self.group.fields[name].attrs.get('loading', 'eager')
             if self in self.group and loading == 'eager':
                 idx = self.group.index(self)
                 length = len(self.group)
                 n = 1
-                while len(id2record) < 80 and (idx - n >= 0 or \
-                        idx + n < length) and n < 100:
+                while len(id2record) < 80 and (idx - n >= 0
+                        or idx + n < length) and n < 100:
                     if idx - n >= 0:
                         record = self.group[idx - n]
                         if name not in record._loaded and record.id >= 0:
@@ -104,17 +109,12 @@ class Record(SignalEvent):
                     value.update(default_values)
                 self.exception = True
             id2value = dict((value['id'], value) for value in values)
-            if len(id2record) > 1:
-                for id, record in id2record.iteritems():
-                    if not record.exception:
-                        record.exception = bool(exception)
-                    value = id2value.get(id)
-                    if record and not record.destroyed and value:
-                        record.set(value, signal=False)
-            else:
-                value = id2value.get(self.id)
-                if value:
-                    self.set(value, signal=False)
+            for id, record in id2record.iteritems():
+                if not record.exception:
+                    record.exception = bool(exception)
+                value = id2value.get(id)
+                if record and not record.destroyed and value:
+                    record.set(value, signal=False)
         return self.group.fields.get(name, False)
 
     def __repr__(self):
@@ -153,14 +153,13 @@ class Record(SignalEvent):
         if value:
             self.signal('record-modified')
 
-    def children_group(self, field_name, check_load=True):
+    def children_group(self, field_name):
         if not field_name:
             return []
-        if check_load:
-            self._check_load([field_name])
+        self._check_load([field_name])
         group = self.value.get(field_name)
         if group is None:
-            return []
+            return None
 
         if id(group.fields) != id(self.group.fields):
             self.group.fields.update(group.fields)
@@ -223,43 +222,29 @@ class Record(SignalEvent):
 
     loaded = property(get_loaded)
 
-    def get(self, get_readonly=True, includeid=False, check_load=True,
-            get_modifiedonly=False):
-        if check_load:
-            self._check_load()
-        value = []
+    def get(self):
+        value = {}
         for name, field in self.group.fields.iteritems():
             if field.attrs.get('readonly'):
                 continue
-            if (field.get_state_attrs(self).get('readonly', False)
-                    and not get_readonly):
+            if field.name not in self.modified_fields and self.id >= 0:
                 continue
-            if (field.name not in self.modified_fields
-                    and get_modifiedonly):
-                continue
-            value.append((name, field.get(self, check_load=check_load,
-                readonly=get_readonly, modified=get_modifiedonly)))
-        value = dict(value)
-        if includeid:
-            value['id'] = self.id
+            value[name] = field.get(self)
         return value
 
-    def get_eval(self, check_load=True):
-        if check_load:
-            self._check_load()
+    def get_eval(self):
         value = {}
         for name, field in self.group.fields.iteritems():
-            value[name] = field.get_eval(self, check_load=check_load)
+            if name not in self._loaded and self.id >= 0:
+                continue
+            value[name] = field.get_eval(self)
         value['id'] = self.id
         return value
 
-    def get_on_change_value(self, check_load=True):
-        if check_load:
-            self._check_load()
+    def get_on_change_value(self):
         value = {}
         for name, field in self.group.fields.iteritems():
-            value[name] = field.get_on_change_value(self,
-                check_load=check_load)
+            value[name] = field.get_on_change_value(self)
         value['id'] = self.id
         return value
 
@@ -277,7 +262,7 @@ class Record(SignalEvent):
         values = self.get()
         try:
             RPCExecute('model', self.model_name, 'pre_validate', values,
-                context=self.context_get())
+                main_iteration=False, context=self.context_get())
         except RPCException:
             return False
         return True
@@ -285,9 +270,10 @@ class Record(SignalEvent):
     def save(self, force_reload=True):
         if self.id < 0 or self.modified:
             if self.id < 0:
-                value = self.get(get_readonly=True)
+                value = self.get()
                 try:
-                    res = RPCExecute('model', self.model_name, 'create', value,
+                    res, = RPCExecute('model', self.model_name, 'create',
+                        [value], main_iteration=False,
                         context=self.context_get())
                 except RPCException:
                     return False
@@ -295,16 +281,15 @@ class Record(SignalEvent):
                 self.id = res
                 self.group.id_changed(old_id)
             elif self.modified:
-                self._check_load()
-                value = self.get(get_readonly=True, get_modifiedonly=True,
-                        check_load=False)
+                value = self.get()
                 if value:
                     context = self.context_get()
                     context = context.copy()
                     context['_timestamp'] = self.get_timestamp()
                     try:
                         RPCExecute('model', self.model_name, 'write',
-                            [self.id], value, context=context)
+                            [self.id], value, main_iteration=False,
+                            context=context)
                     except RPCException:
                         return False
             self._loaded.clear()
@@ -319,7 +304,7 @@ class Record(SignalEvent):
         return self.id
 
     @staticmethod
-    def delete(records, context=None):
+    def delete(records):
         if not records:
             return
         record = records[0]
@@ -328,7 +313,6 @@ class Record(SignalEvent):
         assert all(r.group.root_group == root_group for r in records)
         records = [r for r in records if r.id >= 0]
         ctx = {}
-        ctx.update(context or {})
         ctx['_timestamp'] = {}
         for rec in records:
             ctx['_timestamp'].update(rec.get_timestamp())
@@ -338,18 +322,19 @@ class Record(SignalEvent):
         reload_ids = list(reload_ids)
         try:
             RPCExecute('model', record.model_name, 'delete', list(record_ids),
-                context=ctx)
+                main_iteration=False, context=ctx)
         except RPCException:
             return False
         if reload_ids:
             root_group.reload(reload_ids)
         return True
 
-    def default_get(self, domain=None, context=None):
+    def default_get(self):
         if len(self.group.fields):
             try:
                 vals = RPCExecute('model', self.model_name, 'default_get',
-                    self.group.fields.keys(), context=context)
+                    self.group.fields.keys(), main_iteration=False,
+                    context=self.context_get())
             except RPCException:
                 return
             if (self.parent
@@ -370,7 +355,8 @@ class Record(SignalEvent):
     def rec_name(self):
         try:
             return RPCExecute('model', self.model_name, 'read', [self.id],
-                ['rec_name'], context=self.context_get())[0]['rec_name']
+                ['rec_name'], main_iteration=False,
+                context=self.context_get())[0]['rec_name']
         except RPCException:
             return ''
 
@@ -403,7 +389,7 @@ class Record(SignalEvent):
     def context_get(self):
         return self.group.context
 
-    def set_default(self, val, signal=True, modified=False):
+    def set_default(self, val, signal=True):
         for fieldname, value in val.items():
             if fieldname not in self.group.fields:
                 continue
@@ -414,14 +400,13 @@ class Record(SignalEvent):
                     self.value[field_rec_name] = val[field_rec_name]
                 elif field_rec_name in self.value:
                     del self.value[field_rec_name]
-            self.group.fields[fieldname].set_default(self, value,
-                modified=modified)
+            self.group.fields[fieldname].set_default(self, value)
             self._loaded.add(fieldname)
         self.validate(softvalidation=True)
         if signal:
             self.signal('record-changed')
 
-    def set(self, val, modified=False, signal=True):
+    def set(self, val, signal=True):
         later = {}
         for fieldname, value in val.iteritems():
             if fieldname == '_timestamp':
@@ -441,17 +426,39 @@ class Record(SignalEvent):
                     self.value[field_rec_name] = val[field_rec_name]
                 elif field_rec_name in self.value:
                     del self.value[field_rec_name]
-            self.group.fields[fieldname].set(self, value, modified=False)
+            self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
         for fieldname, value in later.iteritems():
-            self.group.fields[fieldname].set(self, value, modified=False)
+            self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
-        if modified:
-            self.modified_fields.update(dict((x, None) for x in val))
-            self.signal('record-modified')
         if signal:
             self.signal('record-changed')
 
+    def set_on_change(self, values):
+        later = {}
+        for fieldname, value in values.items():
+            if fieldname not in self.group.fields:
+                continue
+            if isinstance(self.group.fields[fieldname], fields.O2MField):
+                later[fieldname] = value
+                continue
+            if isinstance(self.group.fields[fieldname], (fields.M2OField,
+                        fields.ReferenceField)):
+                field_rec_name = fieldname + '.rec_name'
+                if field_rec_name in values:
+                    self.value[field_rec_name] = values[field_rec_name]
+                elif field_rec_name in self.value:
+                    del self.value[field_rec_name]
+            self.group.fields[fieldname].set_on_change(self, value)
+        for fieldname, value in later.items():
+            # on change recursion checking is done only for x2many
+            field_x2many = self.group.fields[fieldname]
+            try:
+                field_x2many.in_on_change = True
+                field_x2many.set_on_change(self, value)
+            finally:
+                field_x2many.in_on_change = False
+
     def reload(self, fields=None):
         if self.id < 0:
             return
@@ -460,32 +467,27 @@ class Record(SignalEvent):
         else:
             for field in fields:
                 self[field]
-        self.validate([])
+        self.validate(fields or [])
 
-    def expr_eval(self, expr, check_load=False):
+    def expr_eval(self, expr):
         if not isinstance(expr, basestring):
             return expr
-        if check_load:
-            self._check_load()
         ctx = rpc.CONTEXT.copy()
         ctx['context'] = ctx.copy()
         ctx['context'].update(self.context_get())
-        for name, field in self.group.fields.items():
-            ctx[name] = field.get_eval(self, check_load=check_load)
-
+        ctx.update(self.get_eval())
         ctx['active_model'] = self.model_name
         ctx['active_id'] = self.id
-        ctx['id'] = self.id  # Force local id
         ctx['_user'] = rpc._USER
         if self.parent and self.parent_name:
             ctx['_parent_' + self.parent_name] = \
-                    common.EvalEnvironment(self.parent, check_load)
+                common.EvalEnvironment(self.parent)
         val = PYSONDecoder(ctx).decode(expr)
         return val
 
     def _get_on_change_args(self, args):
         res = {}
-        values = common.EvalEnvironment(self, True, 'on_change')
+        values = common.EvalEnvironment(self, 'on_change')
         for arg in args:
             scope = values
             for i in arg.split('.'):
@@ -501,33 +503,12 @@ class Record(SignalEvent):
             attr = PYSONDecoder().decode(attr)
         args = self._get_on_change_args(attr)
         try:
-            res = RPCExecute('model', self.model_name,
-                'on_change_' + fieldname, args, context=self.context_get())
+            res = RPCExecute('model', self.model_name, 'on_change_' +
+                fieldname, args, main_iteration=False,
+                context=self.context_get())
         except RPCException:
             return
-        later = {}
-        for fieldname, value in res.items():
-            if fieldname not in self.group.fields:
-                continue
-            if isinstance(self.group.fields[fieldname], fields.O2MField):
-                later[fieldname] = value
-                continue
-            if isinstance(self.group.fields[fieldname], (fields.M2OField,
-                        fields.ReferenceField)):
-                field_rec_name = fieldname + '.rec_name'
-                if field_rec_name in res:
-                    self.value[field_rec_name] = res[field_rec_name]
-                elif field_rec_name in self.value:
-                    del self.value[field_rec_name]
-            self.group.fields[fieldname].set_on_change(self, value)
-        for fieldname, value in later.items():
-            # on change recursion checking is done only for x2many
-            field_x2many = self.group.fields[fieldname]
-            try:
-                field_x2many.in_on_change = True
-                field_x2many.set_on_change(self, value)
-            finally:
-                field_x2many.in_on_change = False
+        self.set_on_change(res)
 
     def on_change_with(self, field_name):
         fieldnames = set()
@@ -547,14 +528,19 @@ class Record(SignalEvent):
                 continue
             fieldnames.add(fieldname)
             values.update(self._get_on_change_args(on_change_with))
+            if isinstance(self.group.fields[fieldname], (fields.M2OField,
+                        fields.ReferenceField)):
+                field_rec_name = fieldname + '.rec_name'
+                if field_rec_name in self.value:
+                    del self.value[field_rec_name]
         if fieldnames:
             try:
                 result = RPCExecute('model', self.model_name, 'on_change_with',
-                    values, list(fieldnames), context=self.context_get())
+                    values, list(fieldnames), main_iteration=False,
+                    context=self.context_get())
             except RPCException:
                 return
-            for fieldname, value in result.items():
-                self.group.fields[fieldname].set_on_change(self, value)
+            self.set_on_change(result)
         for fieldname in later:
             on_change_with = self.group.fields[fieldname].attrs.get(
                     'on_change_with')
@@ -562,7 +548,7 @@ class Record(SignalEvent):
             try:
                 result = RPCExecute('model', self.model_name,
                     'on_change_with_' + fieldname, values,
-                    context=self.context_get())
+                    main_iteration=False, context=self.context_get())
             except RPCException:
                 return
             self.group.fields[fieldname].set_on_change(self, result)
@@ -580,7 +566,8 @@ class Record(SignalEvent):
         args = self._get_on_change_args(autocomplete)
         try:
             res = RPCExecute('model', self.model_name, 'autocomplete_' +
-                fieldname, args, context=self.context_get())
+                fieldname, args, main_iteration=False,
+                context=self.context_get())
         except RPCException:
             # ensure res is a list
             res = []
@@ -595,7 +582,7 @@ class Record(SignalEvent):
                     'search_count', [
                         ('resource', '=',
                             '%s,%s' % (self.model_name, self.id)),
-                        ])
+                        ], main_iteration=False)
             except RPCException:
                 return 0
         return self.attachment_count
