@@ -1,6 +1,7 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 "Screen"
+import copy
 import gobject
 try:
     import simplejson as json
@@ -8,7 +9,8 @@ except ImportError:
     import json
 import collections
 import xml.dom.minidom
-import tryton.rpc as rpc
+import gettext
+
 from tryton.gui.window.view_form.model.group import Group
 from tryton.gui.window.view_form.model.record import Record
 from tryton.gui.window.view_form.view.screen_container import ScreenContainer
@@ -21,6 +23,8 @@ from tryton.common import RPCExecute, RPCException, MODELACCESS, \
     node_attributes, sur
 from tryton.action import Action
 
+_ = gettext.gettext
+
 
 class Screen(SignalEvent):
     "Screen"
@@ -32,7 +36,7 @@ class Screen(SignalEvent):
     def __init__(self, model_name, view_ids=None, mode=None, context=None,
             views_preload=None, domain=None, row_activate=None, limit=None,
             readonly=False, exclude_field=None, sort=None, search_value=None,
-            alternate_view=False):
+            tab_domain=None, alternate_view=False):
         if view_ids is None:
             view_ids = []
         if mode is None:
@@ -57,6 +61,7 @@ class Screen(SignalEvent):
         else:
             self.row_activate = row_activate
         self.domain = domain
+        self.size_limit = None
         self.views_preload = views_preload
         self.model_name = model_name
         self.context = context
@@ -70,7 +75,7 @@ class Screen(SignalEvent):
         self.new_group()
         self.__current_record = None
         self.current_record = None
-        self.screen_container = ScreenContainer()
+        self.screen_container = ScreenContainer(tab_domain)
         self.screen_container.alternate_view = alternate_view
         self.widget = self.screen_container.widget_get()
         self.__current_view = 0
@@ -82,15 +87,9 @@ class Screen(SignalEvent):
             lambda: collections.defaultdict(lambda: None))
         self.domain_parser = None
         self.pre_validate = False
-
-        if mode:
-            self.view_to_load = mode[1:]
-            view_id = False
-            if self.view_ids:
-                view_id = self.view_ids.pop(0)
-            view = self.add_view_id(view_id, mode[0])
-            self.screen_container.set(view.widget)
-        self.display()
+        self.view_to_load = mode[:]
+        if view_ids or mode:
+            self.switch_view()
 
     def __repr__(self):
         return '<Screen %s at %s>' % (self.model_name, id(self))
@@ -105,27 +104,35 @@ class Screen(SignalEvent):
                 except RPCException:
                     return
 
-            fields = self.fields_view_tree['fields']
-            for name, props in fields.iteritems():
-                if props['type'] not in ('selection', 'reference'):
-                    continue
-                if isinstance(props['selection'], (tuple, list)):
-                    continue
-                props['selection'] = self.get_selection(props)
+            if not self.domain_parser:
+                fields = copy.deepcopy(self.fields_view_tree['fields'])
+                for name, props in fields.iteritems():
+                    if props['type'] not in ('selection', 'reference'):
+                        continue
+                    if isinstance(props['selection'], (tuple, list)):
+                        continue
+                    props['selection'] = self.get_selection(props)
 
-            # Filter only fields in XML view
-            xml_dom = xml.dom.minidom.parseString(
-                self.fields_view_tree['arch'])
-            root_node, = xml_dom.childNodes
-            xml_fields = [node_attributes(node).get('name')
-                for node in root_node.childNodes if node.nodeName == 'field']
-            if hasattr(collections, 'OrderedDict'):
-                odict = collections.OrderedDict
-            else:
-                odict = dict
-            fields = odict((name, fields[name]) for name in  xml_fields)
+                # Filter only fields in XML view
+                xml_dom = xml.dom.minidom.parseString(
+                    self.fields_view_tree['arch'])
+                root_node, = xml_dom.childNodes
+                xml_fields = [node_attributes(node).get('name')
+                    for node in root_node.childNodes
+                    if node.nodeName == 'field']
+                if hasattr(collections, 'OrderedDict'):
+                    odict = collections.OrderedDict
+                else:
+                    odict = dict
+                fields = odict((name, fields[name]) for name in xml_fields)
+                if 'id' not in fields:
+                    fields['id'] = {
+                        'string': _('ID'),
+                        'name': 'id',
+                        'type': 'integer',
+                        }
 
-            self.domain_parser = DomainParser(fields)
+                self.domain_parser = DomainParser(fields)
 
             self.screen_container.set_screen(self)
             self.screen_container.show_filter()
@@ -134,8 +141,13 @@ class Screen(SignalEvent):
 
     def get_selection(self, props):
         try:
-            selection = RPCExecute('model', self.model_name,
-                props['selection'])
+            change_with = props.get('selection_change_with')
+            if change_with:
+                selection = RPCExecute('model', self.model_name,
+                    props['selection'], dict((p, None) for p in change_with))
+            else:
+                selection = RPCExecute('model', self.model_name,
+                    props['selection'])
         except RPCException:
             selection = []
         selection.sort(lambda x, y: cmp(x[1], y[1]))
@@ -169,6 +181,11 @@ class Screen(SignalEvent):
                 domain = ['AND', domain, self.domain]
         else:
             domain = self.domain
+
+        tab_domain = self.screen_container.get_tab_domain()
+        if tab_domain:
+            domain = ['AND', domain, tab_domain]
+
         try:
             ids = RPCExecute('model', self.model_name, 'search', domain,
                 self.offset, self.limit, self.sort, context=self.context)
@@ -242,7 +259,8 @@ class Screen(SignalEvent):
         self.signal('record-modified')
 
     def _group_changed(self, group, record):
-        self.display()
+        if not self.parent:
+            self.display()
 
     def __get_current_record(self):
         if (self.__current_record is not None
@@ -295,8 +313,6 @@ class Screen(SignalEvent):
         self.widget = None
 
     def default_row_activate(self):
-        from tryton.action import Action
-
         if (self.current_view.view_type == 'tree' and
                 self.current_view.widget_tree.keyword_open):
             return Action.exec_keyword('tree_open', {
@@ -312,19 +328,20 @@ class Screen(SignalEvent):
     def number_of_views(self):
         return len(self.views) + len(self.view_to_load)
 
-    def switch_view(self, view_type=None, default=True, context=None):
-        if not self.parent and self.modified():
-            return
-        self.current_view.set_value()
-        if (self.current_record and
-                self.current_record not in self.current_record.group):
-            self.current_record = None
-        fields = self.current_view.get_fields()
-        if self.current_record and not self.current_record.validate(fields):
-            self.screen_container.set(self.current_view.widget)
-            self.set_cursor()
-            self.current_view.display()
-            return
+    def switch_view(self, view_type=None):
+        if self.current_view:
+            if not self.parent and self.modified():
+                return
+            self.current_view.set_value()
+            if (self.current_record and
+                    self.current_record not in self.current_record.group):
+                self.current_record = None
+            fields = self.current_view.get_fields()
+            if self.current_record and not self.current_record.validate(fields):
+                self.screen_container.set(self.current_view.widget)
+                self.set_cursor()
+                self.current_view.display()
+                return
         if not view_type or self.current_view.view_type != view_type:
             for i in xrange(self.number_of_views):
                 if len(self.view_to_load):
@@ -338,8 +355,6 @@ class Screen(SignalEvent):
                 elif self.current_view.view_type == view_type:
                     break
         self.screen_container.set(self.current_view.widget)
-        if not self.current_record and self.current_view.view_type == 'form':
-            self.new(default=default, context=context)
         self.current_view.cancel()
         self.display()
         self.set_cursor()
@@ -348,14 +363,15 @@ class Screen(SignalEvent):
         if len(self.view_to_load):
             if self.view_ids:
                 view_id = self.view_ids.pop(0)
-                view_type = self.view_to_load.pop(0)
             else:
-                view_id = False
-                view_type = self.view_to_load.pop(0)
+                view_id = None
+            view_type = self.view_to_load.pop(0)
             self.add_view_id(view_id, view_type)
 
-    def add_view_id(self, view_id, view_type, display=False, context=None):
-        if view_type in self.views_preload:
+    def add_view_id(self, view_id, view_type):
+        if view_id and str(view_id) in self.views_preload:
+            view = self.views_preload[str(view_id)]
+        elif not view_id and view_type in self.views_preload:
             view = self.views_preload[view_type]
         else:
             try:
@@ -363,9 +379,9 @@ class Screen(SignalEvent):
                     view_id, view_type, context=self.context)
             except RPCException:
                 return
-        return self.add_view(view, display, context=context)
+        return self.add_view(view)
 
-    def add_view(self, view, display=False, context=None):
+    def add_view(self, view):
         arch = view['arch']
         fields = view['fields']
 
@@ -391,24 +407,13 @@ class Screen(SignalEvent):
         children_field = view.get('field_childs')
 
         from tryton.gui.window.view_form.view.widget_parse import WidgetParse
-        self.group.add_fields(fields, context=context)
+        self.group.add_fields(fields)
 
         parser = WidgetParse(parent=self.parent)
         view = parser.parse(self, xml_dom, self.group.fields,
                 children_field=children_field)
 
         self.views.append(view)
-
-        if display:
-            self.__current_view = len(self.views) - 1
-            self.screen_container.set(self.current_view.widget)
-            fields = self.current_view.get_fields()
-            if (not self.current_record
-                and self.current_view.view_type == 'form'):
-                self.new()
-            self.set_cursor()
-            self.current_view.cancel()
-            self.display()
         return view
 
     def editable_get(self):
@@ -417,33 +422,21 @@ class Screen(SignalEvent):
                 return self.current_view.widget_tree.editable
         return False
 
-    def new(self, default=True, context=None):
-        if context is None:
-            context = {}
-        if self.current_view and \
-                ((self.current_view.view_type == 'tree' \
-                and not (hasattr(self.current_view.widget_tree, 'editable') \
-                    and self.current_view.widget_tree.editable)) \
-                or self.current_view.view_type == 'graph'):
-            prev_current_record = self.current_record
-            for i in xrange(len(self.views)):
-                self.switch_view()
-                if self.current_view.view_type == 'form':
-                    break
+    def new(self, default=True):
+        if (self.current_view
+                and ((self.current_view.view_type == 'tree'
+                        and not (hasattr(self.current_view.widget_tree,
+                                'editable')
+                            and self.current_view.widget_tree.editable))
+                    or self.current_view.view_type == 'graph')):
+            self.switch_view('form')
             if self.current_view.view_type != 'form':
                 return None
-            if not prev_current_record and self.current_record:
-                # new already called in switch_view
-                return self.current_record
-        ctx = {}
-        ctx.update(rpc.CONTEXT)
-        ctx.update(self.context)
-        ctx.update(context)
         if self.current_record:
             group = self.current_record.group
         else:
             group = self.group
-        record = group.new(default, self.domain, ctx)
+        record = group.new(default)
         group.add(record, self.new_model_position())
         self.current_record = record
         self.display()
@@ -453,9 +446,9 @@ class Screen(SignalEvent):
 
     def new_model_position(self):
         position = -1
-        if self.current_view and self.current_view.view_type == 'tree' \
-                and hasattr(self.current_view.widget_tree, 'editable') \
-                    and self.current_view.widget_tree.editable == 'top':
+        if (self.current_view and self.current_view.view_type == 'tree'
+                and hasattr(self.current_view.widget_tree, 'editable')
+                and self.current_view.widget_tree.editable == 'top'):
             position = 0
         return position
 
@@ -512,20 +505,17 @@ class Screen(SignalEvent):
         elif current_view.view_type in ('tree', 'form'):
             current_view.set_cursor(new=new, reset_view=reset_view)
 
-    def get(self, get_readonly=True, includeid=False, check_load=True,
-            get_modifiedonly=False):
+    def get(self):
         if not self.current_record:
             return None
         self.current_view.set_value()
-        return self.current_record.get(get_readonly=get_readonly,
-                includeid=includeid, check_load=check_load,
-                get_modifiedonly=get_modifiedonly)
+        return self.current_record.get()
 
-    def get_on_change_value(self, check_load=True):
+    def get_on_change_value(self):
         if not self.current_record:
             return None
         self.current_view.set_value()
-        return self.current_record.get_on_change_value(check_load=check_load)
+        return self.current_record.get_on_change_value()
 
     def modified(self):
         if self.current_view.view_type != 'tree':
@@ -661,7 +651,7 @@ class Screen(SignalEvent):
 
     def load(self, ids, set_cursor=True, modified=False):
         self.expanded_nodes.clear()
-        self.group.load(ids, display=False, modified=modified)
+        self.group.load(ids, modified=modified)
         self.current_view.reset()
         if ids:
             self.display(ids[0])
@@ -688,14 +678,14 @@ class Screen(SignalEvent):
             if self.current_view.view_type == 'calendar' and \
                     len(self.views) > 1:
                 self.switch_view()
-            self.search_active(self.current_view.view_type \
-                    in ('tree', 'graph', 'calendar'))
+            self.search_active(self.current_view.view_type
+                in ('tree', 'graph', 'calendar'))
             for view in self.views:
                 view.display()
             self.current_view.widget.set_sensitive(
-                    bool(self.group \
-                            or (self.current_view.view_type != 'form') \
-                            or self.current_record))
+                bool(self.group
+                    or (self.current_view.view_type != 'form')
+                    or self.current_record))
             if set_cursor:
                 self.set_cursor(reset_view=False)
         self.set_tree_state()
@@ -809,7 +799,8 @@ class Screen(SignalEvent):
         if button.get('confirm', False) and not sur(button['confirm']):
             return
         record = self.current_record
-        record.save(force_reload=False)
+        if not record.save(force_reload=False):
+            return
         context = record.context_get()
         try:
             action_id = RPCExecute('model', self.model_name, button['name'],
