@@ -4,7 +4,6 @@ import operator
 from functools import reduce
 import gtk
 import gettext
-import gobject
 from interface import ParserView
 
 _ = gettext.gettext
@@ -21,7 +20,6 @@ class ViewForm(ParserView):
         for widget in self.state_widgets:
             if isinstance(widget, gtk.Button):
                 widget.connect('clicked', self.button_clicked)
-                widget.set_focus_on_click(False)
 
         # Force to display the first time it switches on a page
         # This avoids glitch in position of widgets
@@ -42,7 +40,6 @@ class ViewForm(ParserView):
         vbox = gtk.VBox()
         vp = gtk.Viewport()
         vp.set_shadow_type(gtk.SHADOW_NONE)
-        vp.connect('leave-notify-event', self.leave)
         vp.add(self.widget)
         scroll = gtk.ScrolledWindow()
         scroll.add(vp)
@@ -82,7 +79,9 @@ class ViewForm(ParserView):
                     field = record.group.fields[name]
                     for widget in widgets:
                         if (not focused_widget
-                                or widget.widget.get_focus_child()):
+                                or widget.widget.is_focus()
+                                or (isinstance(widget.widget, gtk.Container)
+                                    and widget.widget.get_focus_child())):
                             widget.set_value(record, field)
 
     def sel_ids_get(self):
@@ -110,13 +109,6 @@ class ViewForm(ParserView):
                         field.get_state_attrs(record)['valid'] = True
                         widget.display(record, field)
 
-    def signal_record_changed(self, *args):
-        for widgets in self.widgets.itervalues():
-            for widget in widgets:
-                if hasattr(widget, 'screen'):
-                    for view in widget.screen.views:
-                        view.signal_record_changed(*args)
-
     def display(self):
         record = self.screen.current_record
         if record:
@@ -126,7 +118,7 @@ class ViewForm(ParserView):
                     for name, field in record.group.fields.iteritems()]
             fields.sort(key=operator.itemgetter(1), reverse=True)
             for field, _ in fields:
-                record[field].get(record, check_load=False)
+                record[field].get(record)
         for name, widgets in self.widgets.iteritems():
             field = None
             if record:
@@ -169,10 +161,6 @@ class ViewForm(ParserView):
                         notebook.set_current_page(i)
             focus_widget.grab_focus()
 
-    def leave(self, widget, event):
-        # leave could be called during event process
-        gobject.idle_add(self.set_value, True)
-
     def button_clicked(self, widget):
         record = self.screen.current_record
         fields = self.get_fields()
