@@ -15,7 +15,6 @@ class Char(WidgetInterface, TranslateMixin):
 
     def __init__(self, field_name, model_name, attrs=None):
         super(Char, self).__init__(field_name, model_name, attrs=attrs)
-        self._default_value = ''
 
         self.widget = gtk.HBox()
         self.autocomplete = bool(attrs.get('autocomplete'))
@@ -35,7 +34,6 @@ class Char(WidgetInterface, TranslateMixin):
 
         focus_entry.set_property('activates_default', True)
         focus_entry.connect('activate', self.sig_activate)
-        focus_entry.connect('focus-in-event', lambda x, y: self._focus_in())
         focus_entry.connect('focus-out-event', lambda x, y: self._focus_out())
         focus_entry.connect('key-press-event', self.send_modified)
         expand, fill = True, True
@@ -57,7 +55,7 @@ class Char(WidgetInterface, TranslateMixin):
 
     @staticmethod
     def translate_widget_set(widget, value):
-        widget.set_text(value)
+        widget.set_text(value or '')
 
     @staticmethod
     def translate_widget_get(widget):
@@ -80,15 +78,18 @@ class Char(WidgetInterface, TranslateMixin):
     def modified(self):
         if self.record and self.field:
             entry = self.entry.get_child() if self.autocomplete else self.entry
-            value = entry.get_text() or self._default_value
+            value = entry.get_text() or ''
             return self.field.get_client(self.record) != value
         return False
 
     def set_value(self, record, field):
         entry = self.entry.get_child() if self.autocomplete else self.entry
-        value = entry.get_text() or self._default_value
+        value = entry.get_text() or ''
         return field.set_client(record, value)
 
+    def get_value(self):
+        return self.entry.get_text()
+
     def display(self, record, field):
         super(Char, self).display(record, field)
         if record and self.autocomplete:
