@@ -19,8 +19,7 @@ class StateMixin(object):
 
     def state_set(self, record):
         if record:
-            state_changes = record.expr_eval(self.attrs.get('states', {}),
-                    check_load=False)
+            state_changes = record.expr_eval(self.attrs.get('states', {}))
         else:
             state_changes = {}
         if state_changes.get('invisible', self.attrs.get('invisible')):
@@ -30,7 +29,16 @@ class StateMixin(object):
 
 
 class Label(StateMixin, gtk.Label):
-    pass
+
+    def state_set(self, record):
+        super(Label, self).state_set(record)
+        if not self.attrs.get('string', True) and 'name' in self.attrs:
+            if record:
+                field = record.group.fields[self.attrs['name']]
+                text = field.get_client(record) or ''
+            else:
+                text = ''
+            self.set_text(text)
 
 
 class VBox(StateMixin, gtk.VBox):
@@ -62,8 +70,7 @@ class Notebook(StateMixin, gtk.Notebook):
     def state_set(self, record):
         super(Notebook, self).state_set(record)
         if record:
-            state_changes = record.expr_eval(self.attrs.get('states', {}),
-                    check_load=False)
+            state_changes = record.expr_eval(self.attrs.get('states', {}))
         else:
             state_changes = {}
         if state_changes.get('readonly', self.attrs.get('readonly')):
@@ -73,25 +80,6 @@ class Notebook(StateMixin, gtk.Notebook):
                     widget.color_set('readonly')
 
 
-class ScrolledWindow(gtk.ScrolledWindow):
-
-    def __init__(self, hadjustment=None, vadjustment=None, attrs=None):
-        super(ScrolledWindow, self).__init__(hadjustment=hadjustment,
-                vadjustment=vadjustment)
-        self.attrs = attrs or {}
-
-    def state_set(self, record):
-        if record:
-            state_changes = record.expr_eval(self.attrs.get('states', {}),
-                    check_load=False)
-        else:
-            state_changes = {}
-        if state_changes.get('invisible', self.attrs.get('invisible')):
-            self.hide()
-        else:
-            self.show()
-
-
 class Alignment(gtk.Alignment):
 
     def __init__(self, widget, attrs):
@@ -259,14 +247,14 @@ class ParserForm(ParserInterface):
                                 and attr_name in fields[attrs['name']].attrs:
                             attrs[attr_name] = fields[attrs['name']
                                     ].attrs[attr_name]
-                    if not text:
+                    if 'string' not in attrs:
                         if gtk.widget_get_default_direction() == \
                                 gtk.TEXT_DIR_RTL:
                             text = _(':') + \
-                                    fields[attrs['name']].attrs['string']
+                                fields[attrs['name']].attrs['string']
                         else:
                             text = fields[attrs['name']].attrs['string'] + \
-                                    _(':')
+                                _(':')
                     if 'xalign' not in attrs:
                         attrs['xalign'] = 1.0
                 elif not text:
@@ -275,9 +263,6 @@ class ParserForm(ParserInterface):
                             text += node.data
                         else:
                             text += node.toxml()
-                if not text:
-                    container.empty_add(int(attrs.get('colspan', 1)))
-                    continue
                 label = Label(text, attrs=attrs)
                 state_widgets.append(label)
                 if CONFIG['client.modepda']:
@@ -360,7 +345,7 @@ class ParserForm(ParserInterface):
                     for attr_name in ('states', 'invisible'):
                         if attr_name in fields[attrs['name']].attrs:
                             attrs[attr_name] = \
-                                    fields[attrs['name']].attrs[attr_name]
+                                fields[attrs['name']].attrs[attr_name]
                     if not text:
                         text = fields[attrs['name']].attrs['string']
                 if not text:
@@ -424,7 +409,8 @@ class ParserForm(ParserInterface):
                 for attr_name in ('relation', 'domain', 'selection',
                         'relation_field', 'string', 'views', 'invisible',
                         'add_remove', 'sort', 'context', 'size', 'filename',
-                        'autocomplete', 'translate', 'create', 'delete'):
+                        'autocomplete', 'translate', 'create', 'delete',
+                        'selection_change_with', 'schema_model'):
                     if attr_name in fields[name].attrs and \
                             not attr_name in attrs:
                         attrs[attr_name] = fields[name].attrs[attr_name]
@@ -478,7 +464,7 @@ class ParserForm(ParserInterface):
                 container.wid_add(frame,
                     colspan=colspan,
                     yexpand=yexpand, yfill=yfill, ypadding=0,
-                    xexpand=xexpand, xfill=xfill,  xpadding=0)
+                    xexpand=xexpand, xfill=xfill, xpadding=0)
             elif node.localName == 'hpaned':
                 hpaned = gtk.HPaned()
                 container.wid_add(hpaned, colspan=int(attrs.get('colspan', 4)),
@@ -527,7 +513,7 @@ class ParserForm(ParserInterface):
                 elif not paned.get_child2():
                     paned.pack2(widget, resize=True, shrink=True)
         return container.pop(), dict_widget, state_widgets, on_write, \
-                notebook_list, cursor_widget
+            notebook_list, cursor_widget
 
 from calendar import Calendar, DateTime, Time
 from float import Float
@@ -547,6 +533,7 @@ from image import Image as Image2
 from progressbar import ProgressBar
 from one2one import One2One
 from richtextbox import RichTextBox
+from dictionary import DictWidget
 
 
 WIDGETS_TYPE = {
@@ -576,4 +563,5 @@ WIDGETS_TYPE = {
     'progressbar': (ProgressBar, 1, False, False),
     'one2one': (One2One, 1, False, False),
     'richtext': (RichTextBox, 1, True, True),
+    'dict': (DictWidget, 1, False, False),
 }
