@@ -31,8 +31,7 @@ def path_convert_id2pos(model, id_path):
         current_id = id_path.pop(0)
         try:
             record = group.get(current_id)
-            group = record.children_group(model.children_field,
-                check_load=True)
+            group = record.children_group(model.children_field)
         except (KeyError, AttributeError):
             return None
     return model.on_get_path(record)
@@ -114,7 +113,7 @@ class AdaptModelGroup(gtk.GenericTreeModel):
     def move_into(self, record, path):
         iter_ = self.get_iter(path)
         parent = self.get_value(iter_, 0)
-        group = parent.children_group(self.children_field, check_load=True)
+        group = parent.children_group(self.children_field)
         if group is not record.group:
             record.group.remove(record, remove=True, force_remove=True)
             # Don't remove record from previous group
@@ -142,8 +141,8 @@ class AdaptModelGroup(gtk.GenericTreeModel):
                 pos += 1
             except KeyError:
                 continue
-        self.group.sort(lambda x, y: \
-                cmp(new_order[ids2pos[x.id]], new_order[ids2pos[y.id]]))
+        self.group.sort(lambda x, y:
+            cmp(new_order[ids2pos[x.id]], new_order[ids2pos[y.id]]))
         prev = None
         for record in self.group:
             if prev:
@@ -195,7 +194,7 @@ class AdaptModelGroup(gtk.GenericTreeModel):
             record = group[i]
             if not self.children_field:
                 break
-            group = record.children_group(self.children_field, check_load=True)
+            group = record.children_group(self.children_field)
         return record
 
     def on_get_value(self, record, column):
@@ -210,6 +209,8 @@ class AdaptModelGroup(gtk.GenericTreeModel):
         if record is None or not self.children_field:
             return False
         children = record.children_group(self.children_field)
+        if children is None:
+            return True
         length = len(children)
         if self.__removed and self.__removed in children:
             length -= 1
@@ -480,7 +481,7 @@ class ViewList(ParserView):
                 cell = self.widget_tree.cells[col.name]
                 if cell.get_textual_value(record) != value:
                     cell.value_from_text(record, value)
-                    if cell.get_textual_value(record) != value:
+                    if value and not cell.get_textual_value(record):
                         # Stop setting value if a value is correctly set
                         idx = len(group)
                         break
@@ -609,10 +610,10 @@ class ViewList(ParserView):
                 parent = record.parent if record else None
                 while parent:
                     if parent.modified:
-                        record = None
                         break
                     parent = parent.parent
-                populate(menu, group.model_name, record)
+                else:
+                    populate(menu, group.model_name, record)
                 for col in self.widget_tree.get_columns():
                     if not col.get_visible() or not col.name:
                         continue
@@ -653,9 +654,9 @@ class ViewList(ParserView):
         screen.load([value])
         encoder = PYSONEncoder()
         act['domain'] = encoder.encode(screen.current_record.expr_eval(
-            act.get('domain', []), check_load=False))
+            act.get('domain', [])))
         act['context'] = encoder.encode(screen.current_record.expr_eval(
-            act.get('context', {}), check_load=False))
+            act.get('context', {})))
         data['model'] = self.screen.model_name
         data['id'] = value
         data['ids'] = [value]
@@ -798,10 +799,10 @@ class ViewList(ParserView):
     # self.widget.set_model(self.store) could be removed if the store
     # has not changed -> better ergonomy. To test
     def display(self):
-        if self.reload \
-                or (not self.widget_tree.get_model()) \
-                    or self.screen.group != \
-                        self.widget_tree.get_model().group:
+        if (self.reload
+                or not self.widget_tree.get_model()
+                or (self.screen.group !=
+                    self.widget_tree.get_model().group)):
             self.store = AdaptModelGroup(self.screen.group,
                     self.children_field)
             self.widget_tree.set_model(self.store)
@@ -836,20 +837,18 @@ class ViewList(ParserView):
                 if not record.get_loaded([child_fieldname]):
                     loaded = False
                     break
-                field_value = record.fields_get()[child_fieldname].get(record,
-                    check_load=False)
+                field_value = record.fields_get()[child_fieldname].get(record)
                 if field_value is not None:
                     value += field_value
                     if record.id in ids or not ids:
                         value_selected += field_value
 
             if loaded:
-                label_str = locale.format(
-                    '%.' + str(self.children[child][3]) + 'f', value_selected,
-                    True)
+                label_str = locale.format('%.*f',
+                    (self.children[child][3], value_selected), True)
                 label_str += ' / '
-                label_str += locale.format(
-                    '%.' + str(self.children[child][3]) + 'f', value, True)
+                label_str += locale.format('%.*f',
+                    (self.children[child][3], value), True)
             else:
                 label_str = '-'
             self.children[child][2].set_text(label_str)
