@@ -7,6 +7,7 @@ import gobject
 from itertools import islice, cycle
 
 from tryton.common import MODELACCESS
+from tryton.common.date_widget import DateEntry
 
 _ = gettext.gettext
 
@@ -85,16 +86,15 @@ class EditableTreeView(TreeView):
 
     def on_create_line(self):
         access = MODELACCESS[self.screen.model_name]
-        if not access['create']:
-            return
         model = self.get_model()
+        if not access['create'] or (self.screen.size_limit is not None
+                and (len(model) >= self.screen.size_limit >= 0)):
+            return
         if self.editable == 'top':
             method = model.prepend
         else:
             method = model.append
-        ctx = self.screen.context.copy()
-        new_record = model.group.new(domain=self.screen.domain,
-                context=ctx)
+        new_record = model.group.new()
         res = method(new_record)
         return res
 
@@ -145,6 +145,8 @@ class EditableTreeView(TreeView):
 
         if event.keyval in self.leaving_events or leaving:
             if isinstance(entry, gtk.Entry):
+                if isinstance(entry, DateEntry):
+                    entry.date_get()
                 txt = entry.get_text()
             else:
                 txt = entry.get_active_text()
@@ -257,6 +259,8 @@ class EditableTreeView(TreeView):
         model = self.get_model()
         record = model.get_value(model.get_iter(path), 0)
         if isinstance(entry, gtk.Entry):
+            if isinstance(entry, DateEntry):
+                entry.date_get()
             self.on_quit_cell(record, column.name, entry.get_text())
         elif isinstance(entry, (gtk.ComboBoxEntry, gtk.ComboBox)):
             self.on_quit_cell(record, column.name, entry.get_active_text())
