@@ -7,7 +7,7 @@ import tempfile
 import gtk
 import locale
 import gettext
-import operator
+import webbrowser
 
 from functools import wraps
 
@@ -22,14 +22,19 @@ from tryton.common import COLORS, node_attributes, \
 import tryton.common as common
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrendererdate import CellRendererDate
-from tryton.common.cellrenderertext import CellRendererText
+from tryton.common.cellrenderertext import CellRendererText, \
+    CellRendererTextCompletion
 from tryton.common.cellrenderertoggle import CellRendererToggle
 from tryton.common.cellrenderercombo import CellRendererCombo
 from tryton.common.cellrendererinteger import CellRendererInteger
 from tryton.common.cellrendererfloat import CellRendererFloat
 from tryton.common.cellrendererbinary import CellRendererBinary
+from tryton.common.cellrendererclickablepixbuf import \
+    CellRendererClickablePixbuf
 from tryton.translate import date_format
 from tryton.common import RPCExecute, RPCException
+from tryton.common.completion import get_completion, update_completion
+from tryton.common.selection import SelectionMixin
 
 _ = gettext.gettext
 
@@ -118,17 +123,19 @@ class ParserTree(ParserInterface):
                 for boolean_fields in ('readonly', 'required', 'expand'):
                     if boolean_fields in node_attrs:
                         node_attrs[boolean_fields] = \
-                                bool(int(node_attrs[boolean_fields]))
+                            bool(int(node_attrs[boolean_fields]))
                 if fname not in fields:
                     continue
                 for attr_name in ('relation', 'domain', 'selection',
                         'relation_field', 'string', 'views', 'invisible',
-                        'add_remove', 'sort', 'context', 'filename'):
+                        'add_remove', 'sort', 'context', 'filename',
+                        'selection_change_with'):
                     if attr_name in fields[fname].attrs and \
                             not attr_name in node_attrs:
                         node_attrs[attr_name] = fields[fname].attrs[attr_name]
-                cell = CELLTYPES.get(node_attrs.get('widget',
-                    fields[fname].attrs['type']))(fname, model_name,
+                cell_type = node_attrs.get('widget',
+                    fields[fname].attrs['type'])
+                cell = CELLTYPES.get(cell_type)(fname, model_name,
                     treeview, node_attrs)
                 treeview.cells[fname] = cell
                 renderer = cell.renderer
@@ -149,26 +156,33 @@ class ParserTree(ParserInterface):
 
                 col = gtk.TreeViewColumn(fields[fname].attrs['string'])
 
+                prefixes = []
+                suffixes = []
+                if cell_type in ('url', 'email', 'callto', 'sip'):
+                    prefixes.append(Affix(fname, self.treeview, node_attrs,
+                            protocol=cell_type))
                 if 'icon' in node_attrs:
-                    render_pixbuf = gtk.CellRendererPixbuf()
-                    col.pack_start(render_pixbuf, expand=False)
-                    icon = node_attrs['icon']
+                    prefixes.append(Affix(fname, self.treeview, node_attrs))
+                for affix in node.childNodes:
+                    affix_attrs = node_attributes(affix)
+                    if affix.localName == 'prefix':
+                        list_ = prefixes
+                    else:
+                        list_ = suffixes
+                    list_.append(Affix(fname, self.treeview, affix_attrs))
 
-                    def setter(column, cell, store, iter):
-                        if (hasattr(self.treeview, 'get_realized')
-                                and not self.treeview.get_realized()):
-                            return
-                        record = store.get_value(iter, 0)
-                        value = record[icon].get_client(record) or ''
-                        common.ICONFACTORY.register_icon(value)
-                        pixbuf = treeview.render_icon(stock_id=value,
-                                size=gtk.ICON_SIZE_BUTTON, detail=None)
-                        cell.set_property('pixbuf', pixbuf)
-                    col.set_cell_data_func(render_pixbuf, setter)
+                for prefix in prefixes:
+                    col.pack_start(prefix.renderer, expand=False)
+                    col.set_cell_data_func(prefix.renderer, prefix.setter)
 
                 col.pack_start(renderer, expand=True)
+                col.set_cell_data_func(renderer, cell.setter)
                 col.name = fname
 
+                for suffix in suffixes:
+                    col.pack_start(suffix.renderer, expand=False)
+                    col.set_cell_data_func(suffix.renderer, suffix.setter)
+
                 hbox = gtk.HBox(False, 2)
                 label = gtk.Label(fields[fname].attrs['string'])
                 label.show()
@@ -186,7 +200,6 @@ class ParserTree(ParserInterface):
                 col.set_widget(hbox)
 
                 col._type = fields[fname].attrs['type']
-                col.set_cell_data_func(renderer, cell.setter)
                 col.set_clickable(True)
                 twidth = {
                     'integer': 60,
@@ -229,7 +242,7 @@ class ParserTree(ParserInterface):
                 i = treeview.append_column(col)
                 if 'sum' in node_attrs and fields[fname].attrs['type'] \
                         in ('integer', 'biginteger', 'float', 'numeric',
-                                'float_time'):
+                            'float_time'):
                     label = gtk.Label(node_attrs['sum'] + _(': '))
                     label_sum = gtk.Label()
                     if isinstance(fields[fname].attrs.get('digits'),
@@ -285,11 +298,66 @@ class ParserTree(ParserInterface):
 
     def set_selection(self, treeview, direction):
         selection = treeview.get_selection()
-        if len(treeview.get_model()):
+        if len(treeview.get_model()) and not selection.count_selected_rows():
             selection.select_path(0)
         return False
 
 
+class Affix(object):
+
+    def __init__(self, field_name, treeview, attrs, protocol=None):
+        super(Affix, self).__init__()
+        self.field_name = attrs.get('name', field_name)
+        self.attrs = attrs
+        self.protocol = protocol
+        self.icon = attrs.get('icon')
+        if protocol:
+            self.renderer = CellRendererClickablePixbuf()
+            self.renderer.connect('clicked', self.clicked)
+            if not self.icon:
+                self.icon = 'tryton-web-browser'
+        elif self.icon:
+            self.renderer = gtk.CellRendererPixbuf()
+        else:
+            self.renderer = gtk.CellRendererText()
+        self.treeview = treeview
+
+    @realized
+    def setter(self, column, cell, store, iter_):
+        record = store.get_value(iter_, 0)
+        field = record[self.field_name]
+        field.state_set(record, states=('invisible',))
+        invisible = field.get_state_attrs(record).get('invisible', False)
+        cell.set_property('visible', not invisible)
+        if self.icon:
+            if self.icon in record.group.fields:
+                value = record[self.icon].get_client(record) or ''
+            else:
+                value = self.icon
+            common.ICONFACTORY.register_icon(value)
+            pixbuf = self.treeview.render_icon(stock_id=value,
+                size=gtk.ICON_SIZE_BUTTON, detail=None)
+            cell.set_property('pixbuf', pixbuf)
+        else:
+            text = self.attrs.get('string', '')
+            if not text:
+                text = field.get_client(record) or ''
+            cell.set_property('text', text)
+
+    def clicked(self, renderer, path):
+        store = self.treeview.get_model()
+        record = store.get_value(store.get_iter(path), 0)
+        value = record[self.field_name].get(record)
+        if value:
+            if self.protocol == 'email':
+                value = 'mailto:%s' % value
+            elif self.protocol == 'callto':
+                value = 'callto:%s' % value
+            elif self.protocol == 'sip':
+                value = 'sip:%s' % value
+            webbrowser.open(value, new=2)
+
+
 class Char(object):
 
     def __init__(self, field_name, model_name, treeview, attrs=None):
@@ -311,7 +379,7 @@ class Char(object):
         else:
             cell.set_sensitive(not (record.deleted or record.removed))
             if isinstance(cell,
-                (CellRendererText, CellRendererDate, CellRendererCombo)):
+                    (CellRendererText, CellRendererDate, CellRendererCombo)):
                 cell.set_property('strikethrough', record.deleted)
             cell.set_property('text', text)
             fg_color = self.get_color(record)
@@ -341,7 +409,8 @@ class Char(object):
 
         if hasattr(self.treeview, 'editable') \
                 and self.treeview.editable:
-            readonly = field.get_state_attrs(record).get('readonly', False)
+            readonly = self.attrs.get('readonly',
+                field.get_state_attrs(record).get('readonly', False))
             if invisible:
                 readonly = True
 
@@ -371,7 +440,7 @@ class Char(object):
         cell.set_property('xalign', align)
 
     def get_color(self, record):
-        return record.expr_eval(self.treeview.colors, check_load=False)
+        return record.expr_eval(self.treeview.colors)
 
     def open_remote(self, record, create, changed=False, text=None,
             callback=None):
@@ -391,6 +460,12 @@ class Char(object):
     def editing_started(self, cell, editable, path):
         return False
 
+    def _get_record_field(self, path):
+        store = self.treeview.get_model()
+        record = store.get_value(store.get_iter(path), 0)
+        field = record.group.fields[self.field_name]
+        return record, field
+
 
 class Int(Char):
 
@@ -413,13 +488,26 @@ class Boolean(Int):
         store = self.treeview.get_model()
         record = store.get_value(store.get_iter(path), 0)
         field = record[self.field_name]
-        if not field.get_state_attrs(record).get('readonly', False):
+        if not self.attrs.get('readonly',
+                field.get_state_attrs(record).get('readonly', False)):
             value = record[self.field_name].get_client(record)
             record[self.field_name].set_client(record, int(not value))
             self.treeview.set_cursor(path)
         return True
 
 
+class URL(Char):
+
+    @realized
+    def setter(self, column, cell, store, iter):
+        super(URL, self).setter(column, cell, store, iter)
+        record = store.get_value(iter, 0)
+        field = record[self.field_name]
+        field.state_set(record, states=('readonly',))
+        readonly = field.get_state_attrs(record).get('readonly', False)
+        cell.set_property('visible', not readonly)
+
+
 class Date(Char):
 
     def __init__(self, field_name, model_name, treeview, attrs=None):
@@ -523,12 +611,6 @@ class Binary(Char):
         size = record[self.field_name].get_size(record)
         cell.set_property('size', common.humanize(size) if size else '')
 
-    def _get_record_field(self, path):
-        store = self.treeview.get_model()
-        record = store.get_value(store.get_iter(path), 0)
-        field = record.group.fields[self.field_name]
-        return record, field
-
     def new_binary(self, renderer, path):
         filename = file_selection(_('Open...'))
         record, field = self._get_record_field(path)
@@ -575,6 +657,13 @@ class Binary(Char):
 
 class M2O(Char):
 
+    def __init__(self, field_name, model_name, treeview, attrs=None):
+        super(M2O, self).__init__(field_name, model_name, treeview,
+            attrs=attrs)
+        if int(attrs.get('completion', 1)):
+            self.renderer = CellRendererTextCompletion(self.set_completion)
+            self.renderer.connect('editing-started', self.editing_started)
+
     def value_from_text(self, record, text, callback=None):
         field = record.group.fields[self.field_name]
         if not text:
@@ -671,6 +760,57 @@ class M2O(Char):
         WinSearch(relation, search_callback, sel_multi=False, ids=ids,
             context=context, domain=domain)
 
+    def set_completion(self, entry, path):
+        if entry.get_completion():
+            entry.set_completion(None)
+        completion = get_completion()
+        completion.connect('match-selected', self._completion_match_selected,
+            path)
+        completion.connect('action-activated',
+            self._completion_action_activated, path)
+        entry.set_completion(completion)
+        entry.connect('key-press-event', self._key_press, path)
+        entry.connect('changed', self._update_completion, path)
+
+    def _key_press(self, entry, event, path):
+        record, field = self._get_record_field(path)
+        if (field.get(record) is not None
+                and event.keyval in (gtk.keysyms.Delete,
+                    gtk.keysyms.BackSpace)):
+            entry.set_text('')
+            field.set_client(record, None)
+        return False
+
+    def _completion_match_selected(self, completion, model, iter_, path):
+        record, field = self._get_record_field(path)
+        rec_name, record_id = model.get(iter_, 0, 1)
+        field.set_client(record, (record_id, rec_name))
+
+        completion.get_entry().set_text(rec_name)
+        completion_model = completion.get_model()
+        completion_model.clear()
+        completion_model.search_text = rec_name
+        return True
+
+    def _update_completion(self, entry, path):
+        record, field = self._get_record_field(path)
+        if field.get(record) is not None:
+            return
+        model = field.attrs['relation']
+        update_completion(entry, record, field, model)
+
+    def _completion_action_activated(self, completion, index, path):
+        record, field = self._get_record_field(path)
+        entry = completion.get_entry()
+
+        def callback():
+            entry.set_text(field.get_client(record))
+        if index == 0:
+            self.open_remote(record, create=False, changed=True,
+                text=entry.get_text(), callback=callback)
+        elif index == 1:
+            self.open_remote(record, create=True, callback=callback)
+
 
 class O2O(M2O):
     pass
@@ -690,8 +830,8 @@ class O2M(Char):
         cell.set_property('xalign', 0.5)
 
     def get_textual_value(self, record):
-        return '( ' + str(len(record[self.field_name].\
-                get_eval(record))) + ' )'
+        return '( ' + str(len(record[self.field_name]
+                .get_eval(record))) + ' )'
 
     def value_from_text(self, record, text, callback=None):
         if callback:
@@ -740,24 +880,14 @@ class M2M(O2M):
             context=context)
 
 
-class Selection(Char):
+class Selection(Char, SelectionMixin):
 
     def __init__(self, *args):
         super(Selection, self).__init__(*args)
         self.renderer = CellRendererCombo()
         self.renderer.connect('editing-started', self.editing_started)
-        self._last_domain = None
-        self._domain_cache = {}
-        selection = self.attrs.get('selection', [])[:]
-        if not isinstance(selection, (list, tuple)):
-            try:
-                selection = RPCExecute('model', self.model_name, selection)
-            except RPCException:
-                selection = []
-        self.selection = selection[:]
-        if self.attrs.get('sort', True):
-            selection.sort(key=operator.itemgetter(1))
-        self.renderer.set_property('model', self.get_model(selection))
+        self.init_selection()
+        self.renderer.set_property('model', self.get_model(self.selection))
         self.renderer.set_property('text-column', 0)
 
     def get_model(self, selection):
@@ -773,7 +903,8 @@ class Selection(Char):
         return model
 
     def get_textual_value(self, record):
-        self.update_selection(record)
+        field = record[self.field_name]
+        self.update_selection(record, field)
         value = record[self.field_name].get(record)
         return dict(self.selection).get(value, '')
 
@@ -787,7 +918,8 @@ class Selection(Char):
         super(Selection, self).editing_started(cell, editable, path)
         store = self.treeview.get_model()
         record = store.get_value(store.get_iter(path), 0)
-        self.update_selection(record)
+        field = record[self.field_name]
+        self.update_selection(record, field)
         model = self.get_model(self.selection)
         editable.set_model(model)
         # GTK 2.24 and above use a ComboBox instead of a ComboBoxEntry
@@ -802,49 +934,14 @@ class Selection(Char):
         completion.set_text_column(0)
         return False
 
-    def update_selection(self, record):
-        if 'relation' not in self.attrs:
-            return
-        field = record[self.field_name]
-        domain = field.domain_get(record)
-        if str(domain) in self._domain_cache:
-            self.selection = self._domain_cache[str(domain)]
-            self._last_domain = domain
-        if domain != self._last_domain:
-            try:
-                result = RPCExecute('model', self.attrs['relation'],
-                    'search_read', domain, 0, None, None, ['rec_name'])
-            except RPCException:
-                result = None
 
-            if isinstance(result, list):
-                selection = [(x['id'], x['rec_name']) for x in result]
-                selection.append((False, ''))
-                self._last_domain = domain
-                self._domain_cache[str(domain)] = selection
-            else:
-                selection = []
-                self._last_domain = None
-        else:
-            selection = self.selection
-        self.selection = selection[:]
-
-
-class Reference(Char):
+class Reference(Char, SelectionMixin):
 
     def __init__(self, field_name, model_name, treeview, attrs=None):
         super(Reference, self).__init__(field_name, model_name, treeview,
             attrs=attrs)
-        self._selection = {}
-        selection = attrs.get('selection', [])
-        if not isinstance(selection, (list, tuple)):
-            try:
-                selection = RPCExecute('model', model_name, selection)
-            except RPCException:
-                selection = []
-        selection.sort(key=operator.itemgetter(1))
-        for i, j in selection:
-            self._selection[i] = str(j)
+        self.init_selection()
+        self._selection = dict(self.selection)
 
     def get_textual_value(self, record):
         value = record[self.field_name].get_client(record)
@@ -888,7 +985,7 @@ class ProgressBar(object):
         value = float(self.get_textual_value(record) or 0.0)
         cell.set_property('value', value)
         digit = field.digits(record)[1]
-        text = locale.format('%.' + str(digit) + 'f', value, True)
+        text = locale.format('%.*f', (digit, value), True)
         cell.set_property('text', text + '%')
 
     def open_remote(self, record, create, changed=False, text=None,
@@ -919,8 +1016,7 @@ class Button(object):
     @realized
     def setter(self, column, cell, store, iter):
         record = store.get_value(iter, 0)
-        states = record.expr_eval(self.attrs.get('states', {}),
-            check_load=False)
+        states = record.expr_eval(self.attrs.get('states', {}))
         invisible = states.get('invisible', False)
         cell.set_property('visible', not invisible)
         readonly = states.get('readonly', False)
@@ -940,7 +1036,7 @@ class Button(object):
         record = store.get_value(store.get_iter(path), 0)
 
         state_changes = record.expr_eval(
-            self.attrs.get('states', {}), check_load=False)
+            self.attrs.get('states', {}))
         if state_changes.get('invisible') \
                 or state_changes.get('readonly'):
             return True
@@ -962,10 +1058,10 @@ CELLTYPES = {
     'time': Time,
     'boolean': Boolean,
     'text': Char,
-    'url': Char,
-    'email': Char,
-    'callto': Char,
-    'sip': Char,
+    'url': URL,
+    'email': URL,
+    'callto': URL,
+    'sip': URL,
     'progressbar': ProgressBar,
     'reference': Reference,
     'one2one': O2O,
