@@ -8,6 +8,7 @@ import collections
 
 import tryton.common as common
 from tryton.common.domain_parser import quote
+from tryton.common.placeholder_entry import PlaceholderEntry
 from tryton.translate import date_format
 from tryton.config import TRYTON_ICON
 from tryton.pyson import PYSONDecoder
@@ -15,32 +16,75 @@ from tryton.pyson import PYSONDecoder
 _ = gettext.gettext
 
 
+class Dates(gtk.HBox):
+
+    def __init__(self, format_):
+        super(Dates, self).__init__()
+        self.from_ = common.date_widget.DateEntry(format_)
+        self.pack_start(self.from_, expand=True, fill=True)
+        self.pack_start(gtk.Label(_('..')), expand=False, fill=False)
+        self.to = common.date_widget.DateEntry(format_)
+        self.pack_start(self.to, expand=True, fill=True)
+
+    def get_value(self):
+        from_ = self.from_.get_text()
+        to = self.to.get_text()
+        if from_ and to:
+            if from_ != to:
+                return '%s..%s' % (quote(from_), quote(to))
+            else:
+                return quote(from_)
+        elif from_:
+            return '>=%s' % quote(from_)
+        elif to:
+            return '<%s' % quote(to)
+
+
+class Selection(gtk.ScrolledWindow):
+
+    def __init__(self, selections):
+        super(Selection, self).__init__()
+        self.treeview = gtk.TreeView()
+        model = gtk.ListStore(gobject.TYPE_STRING)
+        for selection in selections:
+            model.append((selection,))
+        self.treeview.set_model(model)
+
+        column = gtk.TreeViewColumn()
+        cell = gtk.CellRendererText()
+        column.pack_start(cell)
+        column.add_attribute(cell, 'text', 0)
+        self.treeview.append_column(column)
+        self.treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+        self.treeview.set_headers_visible(False)
+        self.add(self.treeview)
+        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+
+    def get_value(self):
+        values = []
+        model, paths = self.treeview.get_selection().get_selected_rows()
+        for path in paths:
+            iter_ = model.get_iter(path)
+            values.append(model.get_value(iter_, 0))
+        return ';'.join(quote(v) for v in values)
+
+
 class ScreenContainer(object):
 
-    def __init__(self):
+    def __init__(self, tab_domain):
         self.viewport = gtk.Viewport()
         self.viewport.set_shadow_type(gtk.SHADOW_NONE)
         self.vbox = gtk.VBox(spacing=3)
-        self.vbox.pack_end(self.viewport)
-        self.filter_vbox = None
-        self.filter_button = None
-        self.but_prev = None
-        self.but_next = None
         self.alternate_viewport = gtk.Viewport()
         self.alternate_viewport.set_shadow_type(gtk.SHADOW_NONE)
         self.alternate_view = False
         self.search_window = None
         self.search_table = None
-
-    def widget_get(self):
-        return self.vbox
-
-    def set_screen(self, screen):
-        self.screen = screen
+        self.last_search_text = ''
+        self.tab_domain = tab_domain or []
 
         tooltips = common.Tooltips()
-        if self.filter_vbox is not None:
-            return
+
         self.filter_vbox = gtk.VBox(spacing=0)
         self.filter_vbox.set_border_width(0)
         hbox = gtk.HBox(homogeneous=False, spacing=0)
@@ -52,7 +96,8 @@ class ScreenContainer(object):
         self.filter_button.connect('toggled', self.search_box)
         hbox.pack_start(self.filter_button, expand=False, fill=False)
 
-        self.search_entry = gtk.Entry()
+        self.search_entry = PlaceholderEntry()
+        self.search_entry.set_placeholder_text(_('Search'))
         self.search_entry.set_alignment(0.0)
         self.completion = gtk.EntryCompletion()
         self.completion.set_model(gtk.ListStore(str))
@@ -67,28 +112,54 @@ class ScreenContainer(object):
         self.search_entry.set_completion(self.completion)
         self.search_entry.connect('key-press-event', self.key_press)
         self.search_entry.connect('focus-in-event', self.focus_in)
+        self.search_entry.connect('icon-press', self.icon_press)
 
         hbox.pack_start(self.search_entry, expand=True, fill=True)
 
-        but_find = gtk.Button()
-        tooltips.set_tip(but_find, _('Find'))
-        but_find.connect('clicked', self.do_search)
-        img_find = gtk.Image()
-        img_find.set_from_stock('tryton-find', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_find.set_alignment(0.5, 0.5)
-        but_find.add(img_find)
-        but_find.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(but_find, expand=False, fill=False)
+        def popup(widget):
+            menu = widget._menu
+            for child in menu.children():
+                menu.remove(child)
+            if not widget.props.active:
+                menu.popdown()
+                return
 
-        but_clear = gtk.Button()
-        tooltips.set_tip(but_clear, _('Clear'))
-        but_clear.connect('clicked', self.clear)
-        img_clear = gtk.Image()
-        img_clear.set_from_stock('tryton-clear', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_clear.set_alignment(0.5, 0.5)
-        but_clear.add(img_clear)
-        but_clear.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(but_clear, expand=False, fill=False)
+            def menu_position(menu):
+                x, y = widget.window.get_origin()
+                widget_allocation = widget.get_allocation()
+                return (
+                    widget_allocation.x + x,
+                    widget_allocation.y + widget_allocation.height + y,
+                    False
+                )
+
+            for id_, name, domain in common.VIEW_SEARCH[
+                    self.screen.model_name]:
+                menuitem = gtk.MenuItem(name)
+                menuitem.connect('activate', self.bookmark_activate, domain)
+                menu.add(menuitem)
+
+            menu.show_all()
+            menu.popup(None, None, menu_position, 0, 0)
+
+        def deactivate(menuitem, togglebutton):
+            togglebutton.props.active = False
+
+        but_bookmark = gtk.ToggleButton()
+        self.but_bookmark = but_bookmark
+        tooltips.set_tip(but_bookmark, _('Show bookmarks of filters'))
+        img_bookmark = gtk.Image()
+        img_bookmark.set_from_stock('tryton-bookmark',
+            gtk.ICON_SIZE_SMALL_TOOLBAR)
+        img_bookmark.set_alignment(0.5, 0.5)
+        but_bookmark.add(img_bookmark)
+        but_bookmark.set_relief(gtk.RELIEF_NONE)
+        menu = gtk.Menu()
+        menu.set_property('reserve-toggle-size', False)
+        menu.connect('deactivate', deactivate, but_bookmark)
+        but_bookmark._menu = menu
+        but_bookmark.connect('toggled', popup)
+        hbox.pack_start(but_bookmark, expand=False, fill=False)
 
         but_prev = gtk.Button()
         self.but_prev = but_prev
@@ -116,22 +187,59 @@ class ScreenContainer(object):
 
         hbox.show_all()
         hbox.set_focus_chain([self.search_entry])
-        self.filter_vbox.pack_start(hbox, expand=True, fill=False)
+        self.filter_vbox.pack_start(hbox, expand=False, fill=False)
 
         hseparator = gtk.HSeparator()
         hseparator.show()
-        self.filter_vbox.pack_start(hseparator, expand=True, fill=False)
+        self.filter_vbox.pack_start(hseparator, expand=False, fill=False)
 
-        self.vbox.pack_start(self.filter_vbox, expand=False, fill=True)
+        if self.tab_domain:
+            self.notebook = gtk.Notebook()
+            self.notebook.props.homogeneous = True
+            self.notebook.set_scrollable(True)
+            for name, domain in self.tab_domain:
+                label = gtk.Label('_' + name)
+                label.set_use_underline(True)
+                self.notebook.append_page(gtk.VBox(), label)
+            self.filter_vbox.pack_start(self.notebook, expand=True, fill=True)
+            self.notebook.show_all()
+            # Set the current page before connecting to switch-page to not
+            # trigger the search a second times.
+            self.notebook.set_current_page(0)
+            self.notebook.get_nth_page(0).pack_end(self.viewport)
+            self.notebook.connect('switch-page', self.switch_page)
+            self.notebook.connect_after('switch-page', self.switch_page_after)
+            filter_expand = True
+        else:
+            self.notebook = None
+            self.vbox.pack_end(self.viewport)
+            filter_expand = False
+
+        self.vbox.pack_start(self.filter_vbox, expand=filter_expand, fill=True)
 
         self.but_next.set_sensitive(False)
         self.but_prev.set_sensitive(False)
 
         tooltips.enable()
 
+    def widget_get(self):
+        return self.vbox
+
+    def set_screen(self, screen):
+        self.screen = screen
+        self.but_bookmark.set_sensitive(
+            bool(common.VIEW_SEARCH[screen.model_name]))
+        self.bookmark_match()
+
     def show_filter(self):
         if self.filter_vbox:
             self.filter_vbox.show()
+        if self.notebook:
+            self.notebook.set_show_tabs(True)
+            if self.viewport in self.vbox.get_children():
+                self.vbox.remove(self.viewport)
+                self.notebook.get_nth_page(self.notebook.get_current_page()
+                    ).pack_end(self.viewport)
 
     def hide_filter(self):
         if self.filter_vbox:
@@ -139,6 +247,12 @@ class ScreenContainer(object):
         if self.filter_button and self.filter_button.get_active():
             self.filter_button.set_active(False)
             self.filter_button.toggled()
+        if self.notebook:
+            self.notebook.set_show_tabs(False)
+            if self.viewport not in self.vbox.get_children():
+                self.notebook.get_nth_page(self.notebook.get_current_page()
+                    ).remove(self.viewport)
+                self.vbox.pack_end(self.viewport)
 
     def set(self, widget):
         if self.alternate_view:
@@ -164,12 +278,47 @@ class ScreenContainer(object):
 
     def clear(self, widget=None):
         self.search_entry.set_text('')
+        self.bookmark_match()
 
     def get_text(self):
         return self.search_entry.get_text().decode('utf-8')
 
     def set_text(self, value):
-        return self.search_entry.set_text(value)
+        self.search_entry.set_text(value)
+        self.bookmark_match()
+
+    def bookmark_activate(self, menuitem, domain):
+        self.set_text(self.screen.domain_parser.string(domain))
+        self.do_search()
+
+    def bookmark_match(self):
+        current_text = self.get_text()
+        current_domain = self.screen.domain_parser.parse(current_text)
+        self.search_entry.set_icon_activatable(gtk.ENTRY_ICON_SECONDARY,
+            bool(current_text))
+        self.search_entry.set_icon_sensitive(gtk.ENTRY_ICON_SECONDARY,
+            bool(current_text))
+        icon_stock = self.search_entry.get_icon_stock(gtk.ENTRY_ICON_SECONDARY)
+        for id_, name, domain in common.VIEW_SEARCH[self.screen.model_name]:
+            text = self.screen.domain_parser.string(domain)
+            domain = self.screen.domain_parser.parse(text.decode('utf-8'))
+            if (text == current_text
+                    or domain == current_domain):
+                if icon_stock != 'tryton-star':
+                    self.search_entry.set_icon_from_stock(
+                        gtk.ENTRY_ICON_SECONDARY, 'tryton-star')
+                self.search_entry.set_icon_tooltip_text(
+                    gtk.ENTRY_ICON_SECONDARY, _('Remove this bookmark'))
+                return id_
+        if icon_stock != 'tryton-unstar':
+            self.search_entry.set_icon_from_stock(gtk.ENTRY_ICON_SECONDARY,
+                'tryton-unstar')
+        if current_text:
+            self.search_entry.set_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY,
+                _('Bookmark this filter'))
+        else:
+            self.search_entry.set_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY,
+                None)
 
     def search_next(self, widget=None):
         self.screen.search_next(self.get_text())
@@ -177,6 +326,25 @@ class ScreenContainer(object):
     def search_prev(self, widget=None):
         self.screen.search_prev(self.get_text())
 
+    def switch_page(self, notebook, page, page_num):
+        current_page = notebook.get_nth_page(notebook.get_current_page())
+        current_page.remove(self.viewport)
+
+        new_page = notebook.get_nth_page(page_num)
+        new_page.pack_end(self.viewport)
+
+    def switch_page_after(self, notebook, page, page_num):
+        self.do_search()
+        notebook.grab_focus()
+
+    def get_tab_domain(self):
+        if not self.notebook:
+            return []
+        idx = self.notebook.get_current_page()
+        if idx < 0:
+            return []
+        return self.tab_domain[idx][1]
+
     def match_selected(self, completion, model, iter):
         def callback():
             self.update()
@@ -196,6 +364,29 @@ class ScreenContainer(object):
 
     def key_press(self, widget, event):
         gobject.idle_add(self.update)
+        gobject.idle_add(self.bookmark_match)
+
+    def icon_press(self, widget, icon_pos, event):
+        if icon_pos == 1:
+            icon_stock = self.search_entry.get_icon_stock(icon_pos)
+            model_name = self.screen.model_name
+            if icon_stock == 'tryton-unstar':
+                text = self.get_text()
+                if not text:
+                    return
+                name = common.ask(_('Bookmark Name:'))
+                if not name:
+                    return
+                domain = self.screen.domain_parser.parse(text)
+                common.VIEW_SEARCH.add(model_name, name, domain)
+                self.set_text(self.screen.domain_parser.string(domain))
+            elif icon_stock == 'tryton-star':
+                id_ = self.bookmark_match()
+                common.VIEW_SEARCH.remove(model_name, id_)
+            # Refresh icon and bookmark button
+            self.bookmark_match()
+            self.but_bookmark.set_sensitive(
+                bool(common.VIEW_SEARCH[model_name]))
 
     def focus_in(self, widget, event):
         self.update()
@@ -213,12 +404,15 @@ class ScreenContainer(object):
             text = ''
             for label, entry in self.search_table.fields:
                 if isinstance(entry, gtk.ComboBox):
-                    value = entry.get_active_text()
+                    value = quote(entry.get_active_text())
+                elif isinstance(entry, (Dates, Selection)):
+                    value = entry.get_value()
                 else:
-                    value = entry.get_text()
+                    value = quote(entry.get_text())
                 if value:
-                    text += label + ' ' + quote(value) + ' '
+                    text += label + ' ' + value + ' '
             self.set_text(text)
+            self.last_search_text = self.get_text()
             self.do_search()
 
         def date_activate(entry):
@@ -245,6 +439,7 @@ class ScreenContainer(object):
                         collections.OrderedDict)):
                 fields.sort(key=operator.itemgetter('string'))
             self.search_table = gtk.Table(rows=len(fields), columns=2)
+            self.search_table.set_homogeneous(False)
             self.search_table.set_border_width(5)
             self.search_table.set_row_spacings(2)
             self.search_table.set_col_spacings(2)
@@ -254,19 +449,21 @@ class ScreenContainer(object):
             for i, field in enumerate(fields):
                 label = gtk.Label(field['string'])
                 label.set_alignment(0.0, 0.5)
-                self.search_table.attach(label, 0, 1, i, i + 1)
-                if field['type'] in ('boolean', 'selection'):
+                self.search_table.attach(label, 0, 1, i, i + 1, yoptions=False)
+                yoptions = False
+                if field['type'] == 'boolean':
                     if hasattr(gtk, 'ComboBoxText'):
                         entry = gtk.ComboBoxText()
                     else:
                         entry = gtk.combo_box_new_text()
                     entry.append_text('')
-                    if field['type'] == 'boolean':
-                        selections = (_('True'), _('False'))
-                    else:
-                        selections = tuple(x[1] for x in field['selection'])
+                    selections = (_('True'), _('False'))
                     for selection in selections:
                         entry.append_text(selection)
+                elif field['type'] == 'selection':
+                    selections = tuple(x[1] for x in field['selection'])
+                    entry = Selection(selections)
+                    yoptions = gtk.FILL | gtk.EXPAND
                 elif field['type'] in ('date', 'datetime', 'time'):
                     if field['type'] == 'date':
                         format_ = date_format()
@@ -274,12 +471,14 @@ class ScreenContainer(object):
                         format_ = PYSONDecoder({}).decode(field['format'])
                         if field['type'] == 'datetime':
                             format_ = date_format() + ' ' + format_
-                    entry = common.date_widget.DateEntry(format_)
-                    entry.connect('activate', date_activate)
+                    entry = Dates(format_)
+                    entry.from_.connect('activate', date_activate)
+                    entry.to.connect('activate', date_activate)
                 else:
                     entry = gtk.Entry()
                     entry.connect('activate', lambda *a: search())
-                self.search_table.attach(entry, 1, 2, i, i + 1)
+                self.search_table.attach(entry, 1, 2, i, i + 1,
+                    yoptions=yoptions)
                 self.search_table.fields.append((field['string'] + ':', entry))
 
             scrolled = gtk.ScrolledWindow()
@@ -336,6 +535,21 @@ class ScreenContainer(object):
             if page and self.search_window not in page.dialogs:
                 page.dialogs.append(self.search_window)
             self.search_window.show()
+
+            if self.last_search_text.strip() != self.get_text().strip():
+                for label, entry in self.search_table.fields:
+                    if isinstance(entry, gtk.ComboBox):
+                        entry.set_active(-1)
+                    elif isinstance(entry, Dates):
+                        entry.from_.set_text('')
+                        entry.to.set_text('')
+                    elif isinstance(entry, Selection):
+                        entry.treeview.get_selection().unselect_all()
+                    else:
+                        entry.set_text('')
+                if self.search_table.fields:
+                    self.search_table.fields[0][1].grab_focus()
+
         else:
             self.search_window.hide()
             if page and self.search_window in page.dialogs:
