@@ -27,16 +27,12 @@ class WinForm(NoModal):
         self.save_current = save_current
         self.prev_view = self.screen.current_view
         self.screen.screen_container.alternate_view = True
-        switch_new = False
-        if view_type == 'form' and not self.screen.current_record:
-            switch_new = True
         if view_type not in (x.view_type for x in self.screen.views) and \
                 view_type not in self.screen.view_to_load:
-            self.screen.add_view_id(False, view_type, display=True)
-        else:
-            self.screen.switch_view(view_type=view_type, context=context)
-        if new and not switch_new:
-            self.screen.new(context=self.context)
+            self.screen.add_view_id(None, view_type)
+        self.screen.switch_view(view_type=view_type)
+        if new:
+            self.screen.new()
         self.win = gtk.Dialog(_('Link'), self.parent,
                 gtk.DIALOG_DESTROY_WITH_PARENT)
         self.win.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
@@ -50,19 +46,14 @@ class WinForm(NoModal):
         self.accel_group = gtk.AccelGroup()
         self.win.add_accel_group(self.accel_group)
 
-        self.but_cancel = None
         self.but_ok = None
         self.but_new = None
-        if new or self.save_current:
-            icon_cancel = gtk.STOCK_CANCEL
-            self.but_cancel = self.win.add_button(icon_cancel,
-                    gtk.RESPONSE_CANCEL)
-            self.but_cancel.set_focus_on_click(False)
+        self.but_cancel = self.win.add_button(gtk.STOCK_CANCEL,
+            gtk.RESPONSE_CANCEL)
 
         if new and self.many:
             self.but_new = self.win.add_button(gtk.STOCK_NEW,
                 gtk.RESPONSE_ACCEPT)
-            self.but_new.set_focus_on_click(False)
             self.but_new.set_accel_path('<tryton>/Form/New', self.accel_group)
 
         if self.save_current:
@@ -77,7 +68,6 @@ class WinForm(NoModal):
         else:
             self.but_ok = self.win.add_button(gtk.STOCK_OK,
                 gtk.RESPONSE_OK)
-        self.but_ok.set_focus_on_click(False)
         self.but_ok.add_accelerator('clicked', self.accel_group,
             gtk.keysyms.Return, gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
         self.win.set_default_response(gtk.RESPONSE_OK)
@@ -305,7 +295,7 @@ class WinForm(NoModal):
         self.screen.switch_view()
 
     def _sig_new(self, widget):
-        self.screen.new(context=self.context)
+        self.screen.new()
         self.screen.current_view.widget.set_sensitive(True)
 
     def _sig_next(self, widget):
@@ -392,6 +382,7 @@ class WinForm(NoModal):
 
     def close(self, widget):
         widget.emit_stop_by_name('close')
+        self.response(self.win, gtk.RESPONSE_CANCEL)
         return True
 
     def response(self, win, response_id):
@@ -425,10 +416,16 @@ class WinForm(NoModal):
             if response_id == gtk.RESPONSE_ACCEPT:
                 self.new()
                 return
-        if (self.but_cancel
-                and self.screen.current_record
+        if (self.screen.current_record
                 and response_id in cancel_responses):
-            self.screen.group.remove(self.screen.current_record, remove=True)
+            if (self.screen.current_record.id < 0
+                    or self.save_current):
+                self.screen.group.remove(self.screen.current_record,
+                    remove=True)
+            elif self.screen.current_record.modified:
+                self.screen.current_record.cancel()
+                self.screen.current_record.reload()
+                self.screen.current_record.signal('record-changed')
             result = False
         else:
             result = response_id not in cancel_responses
@@ -436,7 +433,7 @@ class WinForm(NoModal):
         self.destroy()
 
     def new(self):
-        self.screen.new(context=self.context)
+        self.screen.new()
         self.screen.current_view.display()
         self.screen.set_cursor(new=True)
         self.many -= 1
